#
# Python Macro Language for Dragon NaturallySpeaking
#   (c) Copyright 1999 by Joel Gould
#   Portions (c) Copyright 1999 by Dragon Systems, Inc.
#
# unittestMonitorfunction.py
# this script tests the monitor functions, which control windows on (multiple)
# monitors.
# note monitorfunctions.py is in the Unimacro release.
# this test can be run standalone, without NatSpeak or NatLink being on or
# activated.
#
#
# Developed by Quintijn Hoogenboom, q.hoogenboom@antenna.nl /
# http://qh.antenna.nl/unimacro
#pylint:disable=R0904, R0913, W0212, C0209, R0915, R0914
import os
import os.path
import unittest
import time
from dtactions import monitorfunctions as mf
import TestCaseWithHelpers

class TestError(Exception):
    pass
thisDir = os.path.dirname(__file__)

#---------------------------------------------------------------------------
class UnittestMonitorfunctions(TestCaseWithHelpers.TestCaseWithHelpers):
    def setUp(self):
        """setup with 2 monitors, primary left,
            NatSpeak taskbar top primary (29 height),
            Windows taskbar left secondary monitor (width 109)
            (you can check your parameters by running monitorfunctions.py itself)
        """
        mf.monitor_info()
        mf.VIRTUAL_SCREEN = [0, 0, 2704, 1050]
        mf.NMON = 2
        mf.BORDERX = mf.BORDERY = 1
        mf.MONITOR_HNDLES = [65537, 65539]
        mf.MONITOR_INFO[65537] = {'Device': '\\\\.\\DISPLAY1',
                'Flags': 1,
                'Monitor': (0, 0, 1680, 1050),
                'Work': (0, 29, 1680, 1050),
                'offsetx': 0,
                'offsety': 29}
        mf.MONITOR_INFO[65539] = {'Device': '\\\\.\\DISPLAY2',
                'Flags': 0,
                'Monitor': (1680, 0, 2704, 768),
                'Work': (1787, 0, 2704, 768),
                'offsetx': 107,
                'offsety': 0}
                   
    def tearDown(self):
        mf.monitor_info()
        
    
    def wait(self, multiple=1):
        time.sleep(0.1*multiple)
    def visibleWait(self, multiple=1):
        time.sleep(0.7*multiple)

    def doTestGetDistanceInDirection(self, RA, BB, angle, expDist, epsilon=0.01):
        """check the result of the function _get_distance_in_direction
        
        (calculating the distance to an edge of the bounding box)
        """
        mess = "doTestGetDistanceInDirection"
        dist = mf._get_distance_in_direction(RA, BB, angle)
        mess1 = "%s: testing distance, angle: %s"% (mess, angle)
        self.assert_equal(expDist, dist,mess1, epsilon=epsilon)
        return dist

    def test_changing_monitor_info(self):
        """this is a vulnerable test, see if changing (fake) monitor data come through
        """
        fake_nmon = 7
        fake_virtual_screen = [0,0,100,100]
        nmon_now = mf.NMON
        v_s_now = mf.VIRTUAL_SCREEN[:]
        mf.fake_monitor_info_for_testing(nmon=7, virtual_screen=fake_virtual_screen)
        expected = fake_nmon
        actual = mf.NMON
        self.assert_equal(expected, actual, "fake monitor info, NMON not as expected")
        expected = fake_virtual_screen
        actual = mf.VIRTUAL_SCREEN
        self.assert_equal(expected, actual, "fake monitor info, VIRTUAL_SCREEN not as expected")
        

    def test_get_distance_in_direction(self):
        RA = [100, 200, 900, 600]
        boundingbox = [0, 0, 1200, 800]
        BB = boundingbox
        funcDistance = self.doTestGetDistanceInDirection
        funcDistance(RA, BB, 0, 200)
        funcDistance(RA, BB, 10, 203.08)
        funcDistance(RA, BB, 30, 230.94)
        funcDistance(RA, BB, 60, 346.41)
        funcDistance(RA, BB, 90, 300)
        funcDistance(RA, BB, 100, 304.63)
        funcDistance(RA, BB, 130, 311.14)
        funcDistance(RA, BB, 160, 212.84)
        funcDistance(RA, BB, 180, 200)
        funcDistance(RA, BB, 190, 203.08)
        funcDistance(RA, BB, 200, 212.84)
        funcDistance(RA, BB, 209, 206.27)
        funcDistance(RA, BB, 210, 200)
        funcDistance(RA, BB, 220.5, 153.98)
        funcDistance(RA, BB, 260, 101.54)
        funcDistance(RA, BB, 270, 100.0)
        funcDistance(RA, BB, 280, 101.54)
        funcDistance(RA, BB, 300, 115.47)
        funcDistance(RA, BB, 325, 174.34)
        funcDistance(RA, BB, 340, 212.84)
        funcDistance(RA, BB, 350, 203.09)
        funcDistance(RA, BB, 359, 200.030)
        funcDistance(RA, BB, 360, 200)
        funcDistance(RA, BB, 361, 200.03)
        
        # testing slow changes (jumps of maximum)
        dist = 200
        for i in range(360):
            newDist = funcDistance(RA, BB, i, expDist=dist,epsilon=10)
            #print 'i: %s dist: %s delta:(%s)'% (i, newDist, newDist - dist)
            dist = newDist
            
        # R outside BB:
        BB = [0, 0, 1000, 800]
        # first quadrant
        funcDistance( [0, -10, 900, 700], BB, 45, 0)
        funcDistance( [0, -10, 900, 700], BB, 89, 0)
        funcDistance( [0, -10, 900, 700], BB, 90, 100)
        
        # second quadrant
        funcDistance( [0, 10, 1000, 700], BB, 90, 0)
        funcDistance( [0, 10, 1000, 700], BB, 100, 0)
        funcDistance( [0, 10, 1000, 700], BB, 170, 0)
        funcDistance( [0, 10, 1000, 700], BB, 180, 100)

        # third quadrant
        funcDistance( [10, 10, 1000, 900], BB, 190, 0)
        funcDistance( [10, 10, 1000, 900], BB, 200, 0)
        funcDistance( [10, 10, 1000, 900], BB, 225, 0)
        funcDistance( [10, 10, 1000, 900], BB, 270, 10)
        
        # fourth quadrant
        funcDistance( [10, -10, 1000, 900], BB, 290, 0)
        funcDistance( [10, -10, 1000, 900], BB, 300, 0)
        funcDistance( [10, -10, 1000, 900], BB, 325, 0)
        funcDistance( [10, -10, 1000, 900], BB, 360, 0)

        for i in range(362):
            # at all edges no distance:
            funcDistance(BB, BB, i, 0, epsilon=0)
        
        
    def doTestNewCoordinatesSameMonitor(self, begin, end, size, width,
                                        positioning, expBegin, expEnd):
        """check the result of the function is resultx, resulty
        """
        mess = "doTestNewCoordinatesSameMonitor"
        begin1, end1 = mf._get_new_coordinates_same_monitor(begin, end, size, width, positioning)
        mess1 = "%s: testing begin of result"% mess
        self.assert_equal(expBegin, begin1, mess1)
        mess1 = "%s: testing end of result"% mess
        self.assert_equal(expEnd, end1, mess1)

    def test_get_new_coordinates_same_monitor(self):
        testname = 'test_get_new_coordinates_same_monitor'
        funcSame = self.doTestNewCoordinatesSameMonitor
        # width same (move)
        funcSame(10, 790, 1000, None, 'left', 0, 780)
        funcSame(10, 790, 1000, None, 'middle', 110, 890)
        funcSame(10, 790, 1000, None, 'right', 220, 1000)
        # useless, nothing changes:
        funcSame(10, 790, 1000, None, 'relative', 10, 790)
        # width 0.5
        funcSame(10, 790, 1000, 0.5, 'left', 0, 500)
        funcSame(10, 790, 1000, 0.5, 'middle', 250, 750)
        funcSame(10, 790, 1000, 0.5, 'right', 500, 1000)
        funcSame(10, 790, 1000, 0.5, 'relative', 22, 522)


    def doTestNewCoordinatesResizeOtherMonitor(self, begin, end, oldSize, newSize,
                                                     expBegin, expEnd):
        """check the result of the function is resultx, resulty
        """
        mess = "doTestNewCoordinatesResizeOtherMonitor"
        begin1, end1 = mf._get_new_coordinates_resize_other_monitor(begin, end, oldSize, newSize)
        mess1 = "%s: testing begin of result"% mess
        self.assert_equal(expBegin, begin1, mess1)
        mess1 = "%s: testing end of result"% mess
        self.assert_equal(expEnd, end1, mess1)


    
    def test_get_new_coordinates_resize_other_monitor(self):
        funcResizeOther = self.doTestNewCoordinatesResizeOtherMonitor
        # resize, 
        # on and off, back and forth:
        funcResizeOther(10, 790, 800, 1200, 15, 1185)
        funcResizeOther(15, 1185, 1200, 800, 10, 790)
     
    def doTestNewCoordinatesFixedOtherMonitor(self, begin, end, oldSize, newSize,
                                                     expBegin, expEnd):
        """check the result of the function is resultx, resulty
        """
        mess = "doTestNewCoordinatesFixedOtherMonitor"
        begin1, end1 = mf._get_new_coordinates_fixed_other_monitor(begin, end, oldSize, newSize)
        mess1 = "%s: testing begin of result"% mess
        self.assert_equal(expBegin, begin1, mess1)
        mess1 = "%s: testing end of result"% mess
        self.assert_equal(expEnd, end1, mess1)

     
    def test_get_new_coordinates_fixed_other_monitor(self):
        
        funcFixedOther = self.doTestNewCoordinatesFixedOtherMonitor
        # fixed 
        # on and off, back and forth:
        funcFixedOther(10, 790, 800, 1200, 210, 990)
        funcFixedOther(210, 990, 1200, 800, 10, 790)


    def doTestMovePixelsAndBack(self, oldRA, amount, direction, expRA, opposite, mess):
        """move pixels one way, test and move back, test
        """
        mi = mf.MONITOR_INFO[65537]
        mess2 = mess + '  expected RA, after moving (PIXEL) amount: %s, direction: %s'% (amount, direction)
        newRA = mf._move_resize_restore_area(oldRA, resize=0, direction=direction,
                                             amount=amount, units='pixels',keepinside=0, keepinsideall=0,
                                             monitor_info=mi)
        self.assert_equal(expRA, newRA, mess2)
        backRA = mf._move_resize_restore_area(newRA, resize=0, direction=opposite,
                                             amount=amount, units='pixels',keepinside=0, keepinsideall=0,
                                             monitor_info=mi)
        mess2 = mess + 'expected RA, after moving (back) (PIXEL) amount: %s, direction: %s'% (amount, opposite)
        self.assert_equal(oldRA, backRA, mess2)
        
    def doTestMovePixels(self, oldRA, amount, direction, expRA, mess):
        """move pixels one way, test
        """
        mi = mf.MONITOR_INFO[65537]
        mess2 = mess + '  expected RA, after moving (PIXEL) amount: %s, direction: %s'% (amount, direction)
        newRA = mf._move_resize_restore_area(oldRA, resize=0, direction=direction,
                                             amount=amount, units='pixels',keepinside=0, keepinsideall=0,
                                             monitor_info=mi)
        self.assert_equal(expRA, newRA, mess2)
        
    def doTestMoveRelative(self, oldRA, amount, direction, expRA, mess):
        """move relative to some side or corner
        """
        mi = mf.MONITOR_INFO[65537]
        mess2 = mess + '  expected RA, after moving (RELATIVE) amount: %s, direction: %s'% (amount, direction)
        newRA = mf._move_resize_restore_area(oldRA, resize=0, direction=direction,
                                             amount=amount, units='relative',keepinside=0, keepinsideall=0,
                                             monitor_info=mi)
        self.assert_equal(expRA, newRA, mess2)
        

    def test_move_resize_restore_area_move_pixels(self):
        """test the function which gives back a new RA
                
        moving an amount of pixels
                
        """
        testname = "test_move_resize_restore_area_move_pixels"
        ##############################
        #### first set of variables,  no resize
        # no resize, 
        oldRA = [10, 20, 790, 580]
        
        # left, right
        #amount, direction, opposite = 5, 'left', 'right'
        self.doTestMovePixelsAndBack([10, 20, 790, 580], 5, 'left', [5, 20, 785, 580], 'right', testname)
        
        #up, down:
        self.doTestMovePixelsAndBack([10, 20, 790, 580], 6, 'up', [10, 14, 790, 574], 'down', testname)
        
        # lefttop, rightbottom:[13, 27, 793, 577]
        self.doTestMovePixels([10, 30, 790, 580], 6, 'lefttop', [8, 24, 788, 574], testname)
        self.doTestMovePixels([10, 30, 790, 580], 60, 'rightbottom', [64, 57, 844, 607], testname)
        self.doTestMovePixels([10, 30, 790, 580], 12, 'leftbottom', [10, 42, 790, 592], testname)
        self.doTestMovePixels([10, 30, 790, 580], 100, 'righttop', [110, 27, 890, 577], testname)

        # relative amounts:
        self.doTestMoveRelative([10, 30, 790, 580], 0.1, 'lefttop', [9, 27, 789, 577], testname)
        self.doTestMoveRelative([10, 30, 790, 580], 0.5, 'lefttop', [5, 15, 785, 565], testname)
        self.doTestMoveRelative([10, 30, 790, 580], 1, 'lefttop', [0, 0, 780, 550], testname)
        
        self.doTestMoveRelative([10, 30, 790, 580], 0.1, 'righttop', [99, 27, 879, 577], testname)
        self.doTestMoveRelative([10, 30, 790, 580], 0.5, 'righttop', [455, 15, 1235, 565], testname)
        self.doTestMoveRelative([10, 30, 790, 580], 1, 'righttop', [900, 0, 1680, 550], testname)

        self.doTestMoveRelative([10, 30, 790, 580], 0.1, 'rightbottom', [99, 74, 879, 624], testname)
        self.doTestMoveRelative([10, 30, 790, 580], 0.5, 'rightbottom', [455, 251, 1235, 801], testname)
        self.doTestMoveRelative([10, 30, 790, 580], 1, 'rightbottom', [900, 471, 1680, 1021], testname)

        self.doTestMoveRelative([10, 30, 790, 580], 0.1, 'leftbottom', [9, 74, 789, 624], testname)
        self.doTestMoveRelative([10, 30, 790, 580], 0.5, 'leftbottom', [5, 251, 785, 801], testname)
        self.doTestMoveRelative([10, 30, 790, 580], 1, 'leftbottom', [0, 471, 780, 1021], testname)


    def doTestAdjustCoordinatesSideCornersMove(self, oldRA, amountx, amounty,
                                                    side_corner, expRA, mess):
        """test the result of an adjustment to one side or corner
        """
        resize = 0
        newRA = mf._adjust_coordinates_side_corners(oldRA, amountx, amounty, resize,
                                                 side_corner=side_corner, min_size=(100, 70))
        mess1 = "doTest_adjust_coordinates_side_corners_move, %s with\namountx: %x, amounty: %s\n" \
                "resize: %s, side_corner: %s"% (mess, amountx, amounty, resize, side_corner)
        mess2 = "%s, TESTING expectedRA"% mess1
        self.assert_equal(expRA, newRA, mess2)
        
    def doTestAdjustCoordinatesSideCornersResizePlus(self, oldRA, amountx, amounty,
                                                    side_corner, expRA, mess):
        """test the result of an adjustment to one side or corner, making larger, resize = 1
        """
        resize = 1
        newRA = mf._adjust_coordinates_side_corners(oldRA, amountx, amounty, resize,
                                                 side_corner=side_corner, min_size=(100, 70))
        mess1 = "doTestAdjustCoordinatesSideCornersResizePlus, %s with\namountx: %x, amounty: %s\n" \
                "resize: %s, side_corner: %s"% (mess, amountx, amounty, resize, side_corner)
        mess2 = "%s, TESTING expectedRA"% mess1
        self.assert_equal(expRA, newRA, mess2)

    def doTestAdjustCoordinatesSideCornersResizeMinus(self, oldRA, amountx, amounty,
                                                    side_corner, expRA, mess):
        """test the result of an adjustment to one side or corner, making smaller, resize = -1
        """
        resize = -1
        newRA = mf._adjust_coordinates_side_corners(oldRA, amountx, amounty, resize,
                                                 side_corner=side_corner, min_size=(100, 70))
        mess1 = "doTestAdjustCoordinatesSideCornersResizePlus, %s with\namountx: %x, amounty: %s\n" \
                "resize: %s, side_corner: %s"% (mess, amountx, amounty, resize, side_corner)
        mess2 = "%s, TESTING expectedRA"% mess1
        self.assert_equal(expRA, newRA, mess2)
        
 
    def test_adjust_coordinates_side_corners(self):
        """test the function which adjusts the RA towards a corner point
                
        """
        testname = "test_adjust_coordinates_side_corners"
        #
        funcMove = self.doTestAdjustCoordinatesSideCornersMove
        funcMove([10, 20, 790, 580], -5, -6, 'lefttop', [5, 14, 785, 574], "moving to lefttop")
        # to lefttop also without giving this direction:
        funcMove([10, 20, 790, 580], -5, -6, None, [5, 14, 785, 574], "moving to lefttop(None given)")
        
        funcMove([10, 20, 790, 580], 5, -6, 'righttop', [15, 14, 795, 574], "moving to righttop")
        # to lefttop also without giving this direction:
        funcMove([10, 20, 790, 580], 5, -6, None, [15, 14, 795, 574], "moving to righttop (None given)")
        
        funcMove([10, 20, 790, 580], 5, 6, 'rightbottom', [15, 26, 795, 586], "moving to rightbottom")
        # to lefttop also without giving this direction:
        funcMove([10, 20, 790, 580], 5, 6, None, [15, 26, 795, 586], "moving to rightbottom (None given)")
        
        funcMove([10, 20, 790, 580], -5, 6, 'leftbottom', [5, 26, 785, 586], "moving to leftbottom")
        # to lefttop also without giving this direction:
        funcMove([10, 20, 790, 580], -5, 6, None, [5, 26, 785, 586], "moving to leftbottom (None given)")
        
        # not exactly in the direction (overshooting possibly)
        funcMove([10, 20, 790, 580], -50, 6, 'leftbottom', [-40, 26, 740, 586], "moving to the left of leftbottom")
        funcMove([10, 20, 790, 580], -5, -60, 'leftbottom', [5, -40, 785, 520], "moving up instead of to leftbottom")
        
        #### second set of variables, with resize = 1 (making window larger)
        funcStretch = self.doTestAdjustCoordinatesSideCornersResizePlus
        funcStretch([10, 20, 790, 580], -5, -6, 'lefttop', [5, 14, 790, 580], "stretching (larger) to lefttop")
        # to lefttop also without giving this direction:
        funcStretch([10, 20, 790, 580], -5, -6, None, [5, 14, 790, 580], "stretching (larger) to lefttop(None given)")
        
        funcStretch([10, 20, 790, 580], 5, -6, 'righttop', [10, 14, 795, 580], "stretching (larger) to righttop")
        # to righttop also without giving this direction:
        funcStretch([10, 20, 790, 580], 5, -6, None,[10, 14, 795, 580], "stretching (larger) to righttop (None given)")
        
        funcStretch([10, 20, 790, 580], 8, 7, 'rightbottom', [10, 20, 798, 587], "stretching (larger) to rightbottom")
        # to rightbottom also without giving this direction:
        funcStretch([10, 20, 790, 580], 8, 7, None, [10, 20, 798, 587], "stretching (larger) to rightbottom (None given)")
        
        funcStretch([10, 20, 790, 580], -3, 9, 'leftbottom', [7, 20, 790, 589], "stretching (larger) to leftbottom")
        # to leftbottom also without giving this direction:
        funcStretch([10, 20, 790, 580], -3, 9, None, [7, 20, 790, 589], "stretching (larger) to leftbottom (None given)")
        
        # not exactly in the direction (overshooting possible, keepinside is checked later)
        funcStretch([10, 20, 790, 580], -50, 6, 'leftbottom', [-40, 20, 790, 586], "stretching (larger) to the left of leftbottom")
        funcStretch([10, 20, 790, 580], -5, -60, 'leftbottom', [5, 20, 790, 520], "stretching (larger) up instead of to leftbottom")
        
        # making size minimal:
        funcStretch([10, 20, 790, 580], 1000, 0, 'lefttop', [690, 20, 790, 580], "resizing too much left side, using min_size[0]")
        funcStretch([10, 20, 790, 580], 0, 700, 'lefttop', [10, 510, 790, 580], "resizing too much top, using min_size[1]")
        funcStretch([10, 20, 790, 580], -1000, 0, 'rightbottom', [10, 20, 110, 580], "resizing too much right side, using min_size[0]")
        funcStretch([10, 20, 790, 580], 0, -700, 'rightbottom', [10, 20, 790, 90], "resizing too much bottom, using min_size[1]")
        # combined:
        funcStretch([10, 20, 790, 580], -1000, 700, 'righttop', [10, 510, 110, 580], "resizing too much righttop, using min_size")
        funcStretch([10, 20, 790, 580], 1000, -700, 'leftbottom', [690, 20, 790, 90], "resizing too much leftbottom, using min_size")
        
        #### third set of variables, with resize = -1 (making window smaller)
        funcShrink = self.doTestAdjustCoordinatesSideCornersResizeMinus
        funcShrink([10, 20, 790, 580], -5, -6, 'lefttop', [5, 14, 790, 580], "adjusting to lefttop (resize -1 ignored)")
        # shrink from rightbottom if not side_corner given:
        funcShrink([10, 20, 790, 580], -5, -6, None, [10, 20, 785, 574], "shrinking (smaller) at rightbottom (None given)")
        
        funcShrink([10, 20, 790, 580], 5, -6, 'righttop', [10, 14, 795, 580], "adjust at righttop  (resize -1 ignored)")
        # to lefttop also without giving this direction:
        funcShrink([10, 20, 790, 580], 5, -6, None, [15, 20, 790, 574], "shrinking (smaller) at leftbottom (None given)")
        
        funcShrink([10, 20, 790, 580], 8, 7, 'rightbottom', [10, 20, 798, 587], "adjust at rightbottom  (resize -1 ignored)")
        # to lefttop also without giving this direction:
        funcShrink([10, 20, 790, 580], 8, 7, None, [18, 27, 790, 580], "shrinking (smaller) at lefttop (None given)")
        
        funcShrink([10, 20, 790, 580], -3, 9, 'leftbottom', [7, 20, 790, 589], "adjust at (resize -1 ignored)")
        # to lefttop also without giving this direction:
        funcShrink([10, 20, 790, 580], -3, 9, None, [10, 29, 787, 580], "shrinking (smaller) at righttop (None given)")
        

        # making size minimal (identical to funcStretch above, because resize is not used when direction is given):
        # combined, see examples with funcStretch.
        funcShrink([10, 20, 790, 580], -1000, 700, 'righttop', [10, 510, 110, 580], "resizing too much righttop, using min_size")
        funcShrink([10, 20, 790, 580], 1000, -700, 'leftbottom', [690, 20, 790, 90], "resizing too much leftbottom, using min_size")

        # shrinking with automatic direcion to keeping min_size:
        funcShrink([10, 20, 790, 580], -1000, 700, None, [10, 510, 110, 580], "shrinking too much direction None, using min_size")
        funcShrink([10, 20, 790, 580], 1000, -700, None, [690, 20, 790, 90], "shrinking too much direction None, using min_size")
        funcShrink([10, 20, 790, 580], 1000, 700, None, [690, 510, 790, 580], "shrinking too much direction None, using min_size")
        funcShrink([10, 20, 790, 580], -1000, -700, None, [10, 20, 110, 90], "shrinking too much direction None, using min_size")
        
        ###############################
        ##### fourth set of variables, with 'center' values, making resize dependent on amountx, amounty
        ## no resize, not important from which side_corner you start
        #resize = 0
        #oldRA = [10, 20, 790, 580]
        #oldWidth, oldHeight = oldRA[2] - oldRA[0], oldRA[3] - oldRA[1]
        #amountx, amounty = -5, -6
        #
        ## most sensible direction:
        #side_corner = 'center'
        #testRA = oldRA[:]
        #newRA = mf._adjust_coordinates_side_corners(oldRA[:], amountx, amounty, resize,
        #                                         side_corner=side_corner, minSize=10)
        #expRA = [13, 23, 788, 576]
        ## assert for testing:
        #mess = '%s, oldRA should be unchanged.'% testname
        #self.assert_equal(testRA, oldRA, mess)
        #
        #messgroup = '%s, resize centered.'% testname
        #mess = '%s, testing expected RA'% messgroup
        #self.assert_equal(expRA, newRA, mess)
        
        # invalid values for side_corner:
        RA = [10, 20, 790, 580]
        func = mf._adjust_coordinates_side_corners
        self.assertRaises(ValueError, func, RA, 0, 0, 0, side_corner='typo')
        self.assertRaises(ValueError, func, RA, 0, 0, 0, side_corner=0)
        self.assertRaises(ValueError, func, RA, 0, 0, 0, side_corner=1)

        # invalid values for resize:
        self.assertRaises(ValueError, func, RA[:], 4, 5, resize='typo', side_corner=None)
        self.assertRaises(ValueError, func, RA[:], 2, 6, resize=1.3, side_corner=None)
        self.assertRaises(ValueError, func, RA[:], 1, 7, resize=-2, side_corner=None)

    def doTestGetAngleDistanceReverse(self, px, py, qx, qy, expAngle, expDist, mess):
        """test angle distance from p towards q and back
        """
        mess1 = "test_get_angle_distance_reverse, %s"% mess
        # first function:

        angle, dist = mf._get_angle_distance(px, py, qx, qy)
        mess2 = '%s, distance'% mess1
        self.assert_equal(expDist, dist, mess2, epsilon=0.01)
        mess2 = '%s, angle'% mess1
        self.assert_equal(expAngle, angle, mess2, epsilon=0.01)
        # second function:
        deltax, deltay = mf._get_deltax_deltay_from_angle_distance(angle, dist)
        expDeltax = qx - px
        expDeltay = qy - py
        mess2 = '%s calculating back, deltax'% mess1
        self.assert_equal(expDeltax, deltax, mess2, epsilon=0.01)
        mess2 = '%s calculating back, deltay'% mess1
        self.assert_equal(expDeltay, deltay, mess2, epsilon=0.01)
        

    def test_get_angle_distance_reverse(self):
        """test the function which calculates angle and distance between two points
        
        plus the reverse function _get_deltax_deltay_from_angle_distance
        
        
        """
        testname = "test_get_angle_distance_reverse"
        self.doTestGetAngleDistanceReverse(50, 50, 200, 0, 71.5650511771, 158.11, "first quadrant")
        self.doTestGetAngleDistanceReverse(50, 700, 1000, 800, 96.0, 955.24, "second quadrant")
        self.doTestGetAngleDistanceReverse(80, 50, 0, 600, 188.27, 555.78, "third quadrant")
        self.doTestGetAngleDistanceReverse(20, 100, 0, 0, 348.7, 101.98, "fourth quadrant")

        self.doTestGetAngleDistanceReverse(0, 100, 0, 0, 0, 100, "point up")
        self.doTestGetAngleDistanceReverse(300, 600, 1000, 600,  90, 700, "point right")
        self.doTestGetAngleDistanceReverse(800, 50, 800, 600,  180, 550, "point down")
        self.doTestGetAngleDistanceReverse(200, 0, 0, 0, 270, 200, "point left")

        self.doTestGetAngleDistanceReverse(1, 1, 1, 1, 0, 0, "no distance")
        
def run():
    print('starting unittestMonitorfunctions')
    unittest.main()
    

if __name__ == "__main__":
    run()



# (unimacro - natlink macro wrapper/extensions)
# (c) copyright 2003 Quintijn Hoogenboom (quintijn@users.sourceforge.net)
#                    Ben Staniford (ben_staniford@users.sourceforge.net)
#                    Bart Jan van Os (bjvo@users.sourceforge.net)
#
# This file is part of a SourceForge project called "unimacro" see
# http://unimacro.SourceForge.net).
#
# "unimacro" is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, see:
# http://www.gnu.org/licenses/gpl.txt
#
# "unimacro" is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; See the GNU General Public License details.
#
# "unimacro" makes use of another SourceForge project "natlink",
# which has the following copyright notice:
#
# Python Macro Language for Dragon NaturallySpeaking
#   (c) Copyright 1999 by Joel Gould
#   Portions (c) Copyright 1999 by Dragon Systems, Inc.
#
# _repeat (was _generic_movement.py)

# Written by: Quintijn Hoogenboom (QH softwaretraining & advies)
# revisionDate = 'August 2003'/febr 2006
"""grammar in which the cursor, the mouse can move select and drag.

This grammar is based on Joel Gould's _mouse.py and Jonathan Epstein's
version of _generic_movement.py.

Also automatic repeating of commands is going to be implemented in this
grammar.

-added continuous search

"""
#pylint:disable=R0904, E1101, R0912, R0915, C0321
#
import time
import os
import os.path
from dtactions.sendkeys import sendkeys
from dtactions.unimacro import unimacroutils
# from dtactions.unimacro.unimacroactions import doAction as action
import unimacro.natlinkutilsbj as natbj
from natlinkcore import natlinkutils
from natlinkcore import natlinktimer
from natlinkcore import natlinkstatus
import natlink
DEBUG = 1
status = natlinkstatus.NatlinkStatus()
stateError = 'invalid state for timer routine'

# For caret movement, this represents the default speed in milliseconds
# between arrow keys

states = ['moving', 'selecting', 'repeating', 'mousing',
          'movingpage', 'selectingpage', 'dragging', 'searching']

defaultSpeed = {
    'normal': (50, 4000),
    'moving': (50, 2000),
    'selecting': (100, 3000),
    'movingpage':  (300, 5000),
    'selectingpage': (500, 6000),
    'repeating': (500, 5000),
    'mousing':  (2, 200),
    'dragging':  (2, 100),
    'searching':  (200, 4000),
    }
SPEED = {}
     
for state in states:
    if state in defaultSpeed:
        ds = defaultSpeed[state]
        if len(ds) == 2:
            rate = (ds[1]/ds[0])**0.25
            
            SPEED[state] = (ds[0], int(ds[0]*rate), int(ds[0]*rate*rate),
                            int(ds[0]*rate*rate*rate), ds[1])
        elif len(ds) == 5:
            SPEED[state] = defaultSpeed[state][:]
        else:
            print(f'no valid speeds for: {state}') 

defaultMousePixels = 1
waitingSpeed = 500
defaultSpeed = 500
minSpeed = 50
# For caret movement, this is the rate change applied when you make it
# faster.  For example, 1.5 is a 50% speed increase.

moveRateChange = 2.0
fastSlowChange = 3.0 
veryChange = 3.0

Counts = ['1','2','3','4','5','6','7','8','9','10','11','12','13',
          '14','15','16','17','18','19','20','25','30','35','40']
 
# flag for displaying all results through the DisplayMessage function
# gives an error in the English version when saying "wait", change therefore
# in "hold on"
showAll = 1

language = unimacroutils.getLanguage()        
normalSet = ['startMoving', 'startMousing', 'startRepeating', 'startSearching']
############################################################################
#
# Here are some of our instisnce variables
#
#   self.inTimer   set when the timer callback in installed
#   self.curSpeed       current movement speed (milliseconds for timer)
#

ancestor = natbj.IniGrammar
class ThisGrammar(ancestor):
    # when we unload the grammar, we must make sure we clear the timer
    # callback so we keep a variable which is set when we currently own
    # the timer callback

    def __init__(self):
        
        self.inTimer = 0
        self.iconState = 0
        self.state = ''
        self.inside = 0
        self.insideCommand = 0
        self.waiting = 0
        self.drag = 0
        ancestor.__init__(self)

    def unload(self):
        if self.inTimer:
            self.cancelMode()
        ancestor.unload(self)

    # This is our grammar.  The rule 'start' is what is normally active.  The
    # rules 'moving' and 'mousing' are used when we are in caret or
    # mouse movement mode.
    name = 'repeat'
    iniIgnoreGrammarLists = ['mouseCount', 'moveCount'] # are set in this module

    if language == 'nld':
        name = 'herhaal'
        gramSpec = """
# these rules are active when we are moving, mousing or repeating:
<moving> exported = (<changeMoving>|<speed>|<moveDir>|<reverse>|
                          <acceleration>|<moveCount>)+ [<endMoving>] |
                     <endMoving>;
<mousing> exported = (<changeMousing>|<speed>|<mouseDir>|<reverse>|
                          <acceleration>|<mouseCount>)+ [<endMousing>] |
                      <endMousing>;
<searching> exported = (<changeSearching>|<speed>|<searchDir>|<reverse>|
                          <acceleration>|<moveCount>)+ [<endSearching>] |
                      <endSearching>;
<repeating> exported = <changeRepeating>;


# here are the rules controlling speeds etc. (inside the timer)        
<changeMoving> = beweeg|selecteer|blok|'ga verder'|'ga door'|'verder'|
                    (begin|einde)[van](regel|document);
<changeMousing> = muis|sleep|'ga verder'|'ga door'|'verder' ;
<changeRepeating> = dummy;
<changeSearching> =  beweeg|selecteer|blok|zoek|'zoek verder'|'ga verder'|'ga door'|'verder';


<endMoving> = stop [bewegen|selecteren|beweeg|blok|selecteer]| OK | wacht | okee;
<endMousing> = stop [muis|muizen|sleep|slepen]| OK | wacht | okee | klik | 'dubbel klik';
<endSearching> = stop [zoek|zoeken]| OK | wacht | okee;
<endRepeating> = stop [herhaal|herhalen] | OK | wacht | okee;

<reverse> = [ga] terug;
<acceleration> = [[heel]veel] (sneller| langzamer);
<speed> = [[heel] erg] (snel|langzaam) | normaal| gestaag;
<moveDir> = [pagina|alinea|regel] (omhoog|omlaag) |
              [woord] (links|rechts);
<moveCount> = {moveCount};
<mouseDir> = omhoog | omlaag |links | rechts;
<searchDir> = omhoog | omlaag | verder | terug;
<mouseCount> = {mouseCount}[stappen|sprongen|pixels|stap|sprong|pixel];
# here are the subrules which deal with caret movement
<startMoving> exported = (BEWEEG|BLOK|SCROLL) (<speed>|<moveDir>|<moveCount>)+;
<startMousing> exported = MUIS (BEWEEG|SLEEP)
                                [<speed>|<mouseDir>|<mouseCount>]+;
<startRepeating> exported = Herhaal dat <speed>;
<startSearching> exported = Zoek continu [<searchDir>|<speed>|<moveCount>]+;
        """
    else:
        # non Dutch:
        gramSpec = """
# these rules  are active when we are moving, mousing or repeating:
<moving> exported = (<changeMoving>|<speed>|<moveDir>|<reverse>|
                          <acceleration>|<moveCount>)+ [<endMoving>] |
                     <endMoving>;
<mousing> exported = (<changeMousing>|<speed>|<mouseDir>|<reverse>|
                          <acceleration>|<mouseCount>)+ [<endMousing>] |
                      <endMousing>;
<searching> exported = (<changeSearching>|<speed>|<searchDir>|<reverse>|
                          <acceleration>|<moveCount>)+ [<endSearching>] |
                      <endSearching>;
<repeating> exported = <changeRepeating>;

# here are the rules controlling speeds etc. (inside the timer)        
<changeMoving> = move|select|'go on'|(begin|end)[of](line|document);
<changeMousing> =  move|drag|'go on'|continue;
<changeSearching> =  move|select|search|'go on'|continue;
<changeRepeating> = dummy;

<endMoving> = stop [moving]| OK | 'hold on' | 'hold it';
<endMousing> = stop [mousing]| OK | 'hold on' | 'hold it' | click | 'double click';
<endSearching> = stop [searching]| OK | 'hold on' | 'hold it'|cancel;
<endRepeating> = stop [repeating|repeat] | OK | 'hold on' | 'hold it'|cancel;

<reverse> = reverse [direction] | [go] back;
<acceleration> = [much] (faster | slower);
<speed> = [very] (fast|slow) | normal | steady;
<moveDir> = [page|paragraph|line] (up | down) |
              [word] (left | right);
<moveCount> = {moveCount};
<mouseDir> = up | down |left | right;
<mouseCount> = {mouseCount};
<searchDir> = forward | back | up | down;
# here are the subrules which deal with caret movement
<startMoving> exported = Start (Moving|Selecting|Scrolling)
                               [<speed>|<moveDir>|<moveCount>]+;
<startMousing> exported = MOUSE (MOVE|DRAG|'Start Moving'|'Start Dragging')
                                [<speed>|<mouseDir>|<mouseCount>]+;
<startRepeating> exported = Repeat That <speed>;
<startSearching> exported = Search continue [<searchDir>|<speed>|<moveCount>]+;
        """

    def initialize(self):
        debugPrint(f'init language: {language}')
        self.load(self.gramSpec, allResults = 1)
        self.activateSet(normalSet, exclusive=0)
        #self.setList('count', Counts)
        self.setList('moveCount', Counts)
        self.setList('mouseCount', Counts)
        self.nPhrase = 1
        self.nTimes = 1
        self.state = ''
        self.inside = 0
        self.insideCommand = 0
        self.waiting = 0
        self.cancelMode()
        self.curDir = ''
        self.curSpeed = 0
        self. mouseSteps = 10
        self. mouseJumps = 10

        self.prevHandle = 0
        self.startScroll = 0
        debugPrint('end of initialise')
            
    # This subroutine cancels any active movement mode

    def cancelMode(self):
        debugPrint('cancelMode')
        natlink.setTrayIcon()

        if self.inTimer: 
            natlinktimer.setTimerCallback(self.onTimer,0)
            self.inTimer = 0
            if self.drag:
                debugPrint('switch of dragging')
                xPos,yPos = natlink.getCursorPos()
                if self.drag == 2:
                    natlink.playEvents([(natlinkutils.wm_rbuttonup,xPos,yPos)])
                else:
                    natlink.playEvents([(natlinkutils.wm_lbuttonup,xPos,yPos)])
                self.drag = 0
            elif self.state == 'searching':
                self.stopSearch()

        self.state = ''
        self.minorState = ''
        self.curDir = ''
        self.Count = 0
        self. mouseSteps = 10
        self.inside = 0
        self.insideCommand = 0
        self.waiting = 0
        self.drag = 0
        self.repeatFlag = 0
        self.hadRepeat = 0
        self.repeatStuff = None
        self.lastResults = []
        print('_repeat, return to normal state, non exclusive')
        self.activateSet(normalSet, exclusive=0)
        
##    def waitMode(self):
##        if self.waiting:
##            debugPrint('in timer, already waiting')
##            return
##        self.waiting = 1
##        natlink.setTimerCallback(self.onTimer, waitingSpeed)
##        
##    def resumeMode(self):
##        if not self.waiting:
##            debugPrint('resume, was not waiting!')
##            return
##        self.waiting = 0
##        self.startNow()g
##
    def onTimer(self):
##        if self.lastClock:
##            diff = int((time.clock() - self.lastClock) * 1000 )
        moduleInfo = natlink.getCurrentModule()
        if natlink.getMicState() == 'on' and moduleInfo[2] == self.moduleInfo[2]:
            if self.inside or self.insideCommand:
                self.setTrayIcon(1)
                return
            if self.waiting:
                self.setTrayIcon(1) 
                return
            if self.doAction():
                return   # this is the only good exit from onTimer!!!!
        self.cancelMode()

    def startNow(self):
        self.moduleInfo = natlink.getCurrentModule()
        k = self.state + self.minorState
        s = self.curSpeed + 2  # very slow =2 -->> 0
        if self.waiting:
            natlinktimer.setTimerCallback(self.onTimer, waitingSpeed, callAtMicOff=self.cancelMode)
        elif k == 'mousing' or self.state == 'dragging':
            speed = SPEED[k][s]
            steps = defaultMousePixels
            print(f'mousing/dragging, speed: {speed}, steps: {steps}')
            if s < minSpeed:
                steps = (minSpeed//speed) + 1
                speed = steps*speed
                if steps < defaultMousePixels:
                    steps = defaultMousePixels
                    speed = speed*defaultMousePixels/steps
                    print(f'enlarge steps: {steps}, new speed: {speed}')
            else:
                speed = s
            debugPrint(f'mouse starting with speed: {speed}, steps: {steps}')
            natlinktimer.setTimerCallback(self.onTimer, speed, callAtMicOff=self.cancelMode)
            self.mouseSteps = steps
        elif k in SPEED:
            debugPrint(f'starting with speed: {SPEED[k][s]}')
            natlinktimer.setTimerCallback(self.onTimer, SPEED[k][s], callAtMicOff=self.cancelMode)
        else:
            debugPrint(f'timer starting with unknown speed for state/minorstate: {k}')
            natlinktimer.setTimerCallback(self.onTimer, defaultSpeed, callAtMicOff=self.cancelMode)
            
        self.inTimer = 1
        
    def doAction(self):
##         debugPrint('start doAction state: %s, dir: %s, count: %s'% \
##                   (self.state, self.curDir, self.Count))
        if self.state in ['moving', 'selecting','searching']:
            if not self.curDir:
                debugPrint('do action, no direction specified')
                return None
            key = 'ext' + self.curDir
            if self.minorState == 'page':
                if self.curDir == 'down':
                    key = 'extpgdn'
                elif self.curDir == 'up':
                    key = 'extpgup'
                else:
                    debugPrint('invalid direction for minorState page: {self.curDir}')
            elif self.minorState in ['word', 'paragraph'] :
                key = 'ctrl+' + key

            if self.waiting and self.Count:
                nowCount = " " + repr(self.Count)
            else:
                nowCount = ""
                
        if self.state == 'moving':
            if self.startScroll:
                keyBefore = ''                
                if self.curDir == 'up': keyBefore = 'extpgup'
                elif self.curDir == 'down': keyBefore = 'extpgdn'
                elif self.curDir == 'left': keyBefore = 'ctrl+extpgdn'
                elif self.curDir == 'right': keyBefore = 'ctrl+extpgup'
                else:
                    debugPrint(f'invalid direction in scrollcommand: {self.curDir}')
                if keyBefore:
                    sendkeys("{"+keyBefore+"}")
                self.startScroll = 0
            self.setTrayIcon(1)
            sendkeys("{"+key+nowCount+"}")
        
        elif self.state == 'selecting':
            self.setTrayIcon(1)
            sendkeys("{shift+"+key+nowCount+"}")
        elif self.state == 'repeating':
            self.setTrayIcon(1)
            self.repeatNow()
        elif self.state == 'searching':
            self.setTrayIcon(1)
            if not self.curDir:
                print(f'searching, no current direction: {self.curDir}, assume down')
                self.curDir = self.getLastSearchDirection() or 'down'
            self.insideCommand = 1
            count = self.Count or 1
            for _ in range(count):
                res = self.searchForText(self.curDir)
                self.curDir = self.getLastSearchDirection()
                if res == -2:
                    # missing search, did cancel mode
                    return None
            unimacroutils.visibleWait()
            self.insideCommand = 0
        elif self.state == 'mousing':
            self.setTrayIcon(1)
            if self.Count:
                self.moveMouse(self.curDir, self.Count*self.mouseJumps)
            else:
                self.moveMouse(self.curDir, self.mouseSteps)
        elif self.state == 'dragging':
            self.setTrayIcon(1)
            if self.Count:
                self.moveMouse(self.curDir, self.Count*self.mouseJumps)
            else:
                self.moveMouse(self.curDir, self.mouseSteps)
        else:
            return None

        self.lastClock = time.time()
        return 1 # good exit

    def gotBegin(self,moduleInfo):
        if self.inTimer:
            self.inside = 1
 
    def gotResultsObject(self,recogType,resObj):
        if recogType == 'reject':
            return
        handle = natlink.getCurrentModule()[2]
        if handle != self.prevHandle:
            # new window, empty lastResults list:
            self.lastResults = []
            self.prevHandle = handle
            self.repeatFlag = 0
        words = resObj.getWords(0)[:]
        if recogType == 'other':
            #print 'resultObject, %s,  repeatFlag: %s' % (words, self.repeatFlag)
            if self.repeatFlag:
                if words != self.repeatStuff:
                    self.cancelMode()
            elif self.inTimer:
                self.cancelMode()
            elif natlink.getCallbackDepth() < 3:
                if self.lastResults and words == self.lastResults[-1]:
                    pass
                else:
                    self.lastResults.append(words)
                    if len(self.lastResults) > 6:
                        self.lastResults = self.lastResults[1:]
                #debugPrint('lastResults: %s' % `self.lastResults`)
            else:
                debugPrint(f'callbackdepth {natlink.getCallbackDepth()}, words: {words}')
            self.inside = 0
            return
##        skipped this, because it interferes with _message:
##        elif recogType == 'reject':
##            if self.inTimer and self.missed:
##                self.doAction()
##                self.startNow()
        if recogType == 'self':
            self.nDir = ''
            self.Count = 0
            self.nSpeed = None
            self.dirState = ''
            debugPrint('---starting phrase')
            debugPrint(f'callbackdepth {natlink.getCallbackDepth()}, words: {words}')
            if showAll:
                self.DisplayMessage(f'<{" ".join(words)}>')
        else:
            print(f'recogtype: {recogType}')
        self.inside = 0
            
    def gotResults_reverse(self,words,fullResults):
        debugPrint(f'reverse: {words}')
        if self.nDir:
            self.flush()
        d = self.curDir
        if d == 'up': d = 'down'
        elif d == 'down': d = 'up'
        elif d == 'right': d = 'left'
        elif d == 'left': d = 'right'
        if d:
            self.nDir = d
            self.dirState = self.minorState

    def gotResults_moveCount(self,words,fullResults):
        for w in words:
            if self.Count: self.flush()
            debugPrint(f'moveCount: {w}')
            self.Count = int(w)

    def gotResults_mouseCount(self,words,fullResults):
        if self.Count: self.flush()
        for w in words:
            debugPrint(f'mouseCount: {w}')
            if w in Counts:
                self.Count = int(w)
            elif self.hasCommon(w, ('pixel', 'pixels')):
                self.mouseJumps = 1
                self.flush()
            elif self.hasCommon(w, ('stap', 'stappen')):
                self.mouseJumps = 10
                self.flush()
            elif self.hasCommon(w, ('sprong', 'sprongen.')):
                self.mouseJumps = 100
                self.flush()

        
    def gotResults_moveDir(self,words,fullResults):
        for w in words:
            print(f'direction: {w}')
            if self.nDir:
                self.flush()
            d = ''
            if w in ['pagina', 'page']: self.dirState = 'page'
            elif w in ['alinea', 'paragraph']: self.dirState = 'paragraph'
            elif w in ['word', 'woord']: self.dirState = 'word'
            elif w in ['up', 'omhoog']: d = 'up'
            elif w in ['down', 'omlaag']: d = 'down'
            elif w in ['right', 'rechts']: d = 'right'
            elif w in ['left', 'links']: d = 'left'
            if d:
                self.nDir = d

    def gotResults_searchDir(self,words,fullResults):
        for w in words:
            debugPrint('direction: {w}')
            if self.nDir:
                self.flush()
            d = ''
            if w in ['omhoog', 'terug', 'back', 'up']: d = 'up'
            elif w in ['forward', 'omlaag', 'verder', 'down']: d = 'down'
            if d:
                self.nDir = d

    def gotResults_mouseDir(self,words,fullResults):
        for w in words:
            debugPrint('mouseDir: {w}')
            if   w in ['up', 'omhoog']: d = 'up'
            elif w in ['down', 'omlaag']: d = 'down'
            elif w in ['right', 'rechts']: d = 'right'
            elif w in ['left', 'links']: d = 'left'
            else:
                print('invalid direction:', w)
                return
            if d != self.curDir:
                self.curDir = d
                self.minorState = ''
            newState = self.minorState
            # to be filled in!
            if newState != self.minorState:
                self.minorState = newState

    def gotResults_endMoving(self,words,fullResults):
        if self.nDir or self.Count or self.nSpeed is not None:
            self.flush()
        debugPrint('end moving: {words}')
        for w in words:
            if w in ['hold','hold on', 'hold it', 'wacht']:
                self.waiting =  1
                debugPrint('waiting mode')
                return
            self.cancelMode()

    def gotResults_endMousing(self,words,fullResults):
        if self.nDir or self.Count or self.nSpeed is not None:
            self.flush()
        debugPrint('end mousing: {words}')
        for w in words:
            if w in ['hold','hold on', 'hold it', 'wacht']:
                self.waiting =  1
                debugPrint('waiting mode')
                return
            self.cancelMode()
            if self.hasCommon(words, ['click', 'klik']):
                natlinkutils.buttonClick()
            elif self.hasCommon(words, ['double click', 'dubbel klik']):
                natlinkutils.buttonClick('left', 2)

    def gotResults_endSearching(self,words,fullResults):
        if self.nDir or self.Count or self.nSpeed is not None:
            self.flush()
        debugPrint('end searching: {words}')
        for w in words:
            if w in ['hold','hold on', 'hold it', 'wacht']:
                self.waiting =  1
                debugPrint('waiting mode')
                return
            self.cancelMode()
            if self.hasCommon(words, ['cancel', 'annuleren']):
                self.searchGoBack()
                    

    def gotResults_endRepeating(self,words,fullResults):
        if self.nDir or self.Count or self.nSpeed is not None:
            self.flush()
        debugPrint('end repeating: {words}')
        for w in words:
            if w in ['hold','hold on', 'hold it', 'wacht']:
                self.waiting =  1
                debugPrint('waiting mode')
                return
            self.cancelMode()


    def gotResults_changeMoving(self,words,fullResults):
        if self.nDir or self.Count or self.nSpeed is not None:
            self.flush()
        b = r = ''
        for w in words:
            if b and r:
                sendkeys("{"+r+b+"}")
                b = r = ''
            if w in ['go on', 'ga door', 'ga verder', 'verder']:
                debugPrint('going on')
                self.waiting = 0
            elif w in ['move', 'beweeg']:
                if self.state != 'selecting':
                    self.state = 'moving'
                self.waiting = 0
            elif w in ['select','selecteer']:
                self.state = 'selecting'
                self.waiting = 0
            elif w in ['begin']:
                b = 'Home'
            elif w in ['einde','end']:
                b = 'End'
            elif w in ['line', 'regel']:
                r = 'ext'
            elif w in ['document']:
                r = 'ctrl+ext'
        if b and r:
            sendkeys("{"+r+b+"}")
            b = r = ''
            
    def gotResults_changeSearching(self,words,fullResults):
        if self.nDir or self.Count or self.nSpeed is not None:
            self.flush()
        for w in words:
            if w in ['go on', 'ga door', 'ga verder', 'verder', 'continue']:
                debugPrint('going on')
                self.waiting = 0
            elif w in ['zoek', 'search']:
                continue
            elif w in ['move', 'beweeg']:
                debugPrint('change to state moving')  
                self.activateSet(['moving'], exclusive = 1)
                self.state = 'moving'

    def gotResults_changeMousing(self,words,fullResults):
        if self.nDir or self.Count or self.nSpeed is not None:
            self.flush()
        for w in words:
            if w in ['go on', 'ga door', 'ga verder', 'verder', 'continue']:
                debugPrint('going on')
                self.waiting = 0
            

    def gotResults_speed(self,words,fullResults):
        if self.Count:
            self.flush()
            
        factor = 1

        for w in words:
            if w in ['heel', 'erg', 'very']:
                factor = factor * 2
            elif w in ['slow', 'langzaam']:
                self.nSpeed = factor 
            elif w in ['fast', 'snel']:
                self.nSpeed = -factor 
            elif w in ['normal', 'normaal', 'steady', 'gestaag']:
                self.nSpeed = 0
            else:
                print('speed, invalid keyword:', w)
        self.nSpeed = max(min(self.nSpeed, 2), -2)
        debugPrint(f'speed words: {repr(words)}, speed: {self.nSpeed}')


    def gotResults_acceleration(self,words,fullResults):
        if self.Count:
            self.flush()
        factor = 1
        self.nSpeed = self.curSpeed        
        for w in words:
            if w in ['much', 'veel', 'very', 'heel']:
                factor = factor * 2
            elif w in ['slower', 'langzamer']:
                diff =  factor
            elif w in ['faster', 'sneller']:
                diff = -factor
            else:
                print('acceleration, invalid keyword:', w)
                debugPrint ('acceleration, invalid keyword: {w}')
        self.nSpeed = self.nSpeed + diff
        self.nSpeed = max(min(self.nSpeed, 2), -2)
        debugPrint(f'acceleration words: {repr(words)}, new speed: {self.nSpeed}')

    def gotResults_startMoving(self,words,fullResults):
        self.activateSet(['moving'], exclusive = 1)
        if self.hasCommon(words, ['BEWEEG', 'Moving']):
            self.state = 'moving'
        elif self.hasCommon(words, ['BLOK', 'Selecting']):
            self.state = 'selecting'
        elif self.hasCommon(words, ['Scrolling', "SCROLL"]):
            self.startScroll = 1 
            self.state = 'moving'

    def gotResults_startSearching(self,words,fullResults):
        self.activateSet(['searching'], exclusive = 1)
        self.curDir = self.getLastSearchDirection()
        self.state = 'searching'
        

    def gotResults_startMousing(self,words,fullResults):
        self.activateSet(['mousing'], exclusive = 1)
        debugPrint(f'start mousing {words}')
        if self.hasCommon(words, ["MUIS",'MOUSE']):
            self.state = 'mousing'
        if self.hasCommon(words, ["SLEEP",'DRAG']):
            self.state = 'dragging'
            xPos,yPos = natlink.getCursorPos()
            if self.hasCommon(words, ["SNELMENU",'RIGHT']):
                self.drag = 2
                natlink.playEvents([(natlinkutils.wm_rbuttondown,xPos,yPos)])
            else:
                self.drag = 1
                natlink.playEvents([(natlinkutils.wm_lbuttondown,xPos,yPos)])

    def gotResults_startRepeating(self,words,fullResults):
        self.state = 'repeating'


    def gotResults_test(self,words,fullResults):
        self.cancelMode()
        print('words in test', words)
        natlink.recognitionMimic(words)

        
    def gotResults_nfrase(self,words,fullResults):
        self.nPhrase = int(words[0])
            
    def gotResults_repeat(self,words,fullResults):
        if words[0] in ['Herhaal', 'Repeat']:
            self.nPhrase = 1
            self.nTimes = 1
        self.hadRepeat = 1

##    def gotResults_again(self,words,fullResults):
##        global hadRepeatGrammar 
##        nAgain = len(words)/2in 3
##        self.hadResult = 1
##        repeatFlag = 1
##        for i in range(nAgain):
##            RepeatNow(nPhrase, nTimes)
##            hadRepeatGrammar = 1
##            if not repeatFlag:
##                return None
            
    def gotResults_count(self,words,fullResults):
        self.nTimes = int(words[0])

    def startAgain(self):
        self.activateSet(['start','again'])

    def gotResults(self,words,fullResults):
        if self.repeatFlag:
            return
        if self.hadRepeat:
            debugPrint('gotResults, hadRepeat aan')
            self.repeatNow()
            self.hadRepeat = 0
            return
        if self.Count or self.nSpeed is not None or self.nDir:
            self.flush()
        if self.state in states:
            self.startNow()
        debugPrint ('end of phrase (gotResults): {words}')
        self.inside = 0 

    def flush(self):
        debugPrint(f'flush, nDir: {self.nDir}, Count: {self.Count}, nSpeed: {self.nSpeed}')
        if self.nDir:
            self.curDir = self.nDir
            self.nDir = ''
            self.minorState = self.dirState
            self.dirState = ''
        if self.Count:
            self.waiting = 1
            self.doAction()
            self.Count = 0
            return

        if self.nSpeed is not None:
            self.curSpeed = self.nSpeed
            self.nSpeed = None
        self.waiting = 0
        self.doAction()
        

    def moveMouse(self,direction,count):
        xPos,yPos = natlink.getCursorPos()
        if direction == 'up': yPos = yPos - count
        elif direction == 'down': yPos = yPos + count
        elif direction == 'left': xPos = xPos - count
        elif direction == 'right': xPos = xPos + count
        xSize,ySize = natlink.getScreenSize()
        xPos = max(xPos, 0)
        if xPos >= xSize: xPos = xSize - 1
        yPos = max(yPos, 0)
        if yPos >= ySize: yPos = ySize - 1
        natlink.playEvents([(natlinkutils.wm_mousemove,xPos,yPos)])


    # This turns on the tray icon depending on the movement direction.
    # self.iconS tate is used to toggle the image to animate the icon.            
    def setTrayIcon(self,toggleIcon):
        if self.state in ['moving', 'selecting', 'mousing','dragging', 'searching']:
            iconName = self.curDir
            toolTip = self.state + ' ' +self.curDir
        elif self.state == 'repeating':
            toolTip = 'repeating commands'
            iconName = os.path.join(iconDirectory, 'repeat')
        else:
            natlink.setTrayIcon()
            print('setTrayIcon, invalid state:', self.state)
            return

        if self.waiting:
            toolTip = 'WAITING'
        
        if not toggleIcon or self.iconState:
            self.iconState = 0
        else:
            self.iconState = 1
            if self.waiting:
                iconName = os.path.join(iconDirectory, 'waiting')
            else:
                iconName = iconName + '2'

##        print 'iconName: %s'% iconName
        try:            
            if iconName[1] == ':':
                # absolute path, attach extension:
                iconName = iconName + '.ico'
        except IndexError:
            print('cannot set tray icon, no iconName (_repeat), try to clear')
            natlink.setTrayIcon()
        try:
            natlink.setTrayIcon(iconName,toolTip,self.onTrayIcon)
        except natlink.NatError:
            print('cannot set tray icon (_repeat), try to clear')
            natlink.setTrayIcon()
                    

    # This is called if the user clicks on the tray icon.  We simply cancel
    # movement in all cases.
## taken from GrammarX:
##    def onTrayIcon(self,message):
##        natlink.setTrayIcon()
##        self.cancelMode()

    def repeatNow(self):
        debugPrint(f'repeatNow: {self.nTimes} times, {self.nPhrase} phrases')
        # put repeat mode ON:
        self.repeatFlag = 1
        itemrange = list(range(len(self.lastResults)))
        if not itemrange:
            self.repeatFlag = 0
            return 
        itemrange = itemrange[-self.nPhrase:]
        #print 'na itemrange:', itemrange
        for _ in range(self.nTimes):
            for i in itemrange:
                if not self.repeatFlag:
                    return # if something else happened
                self.repeatStuff= self.lastResults[i]
                natlink.recognitionMimic(self.repeatStuff)                
            # reset repeat mode:
        self.repeatFlag = 0
        self.repeatStuff = None
# This is a simple utility subroutine.  It takes two lists of words and 
# returns the first word it finds which is in both lists.  We use this to
# extract special words (like the direction) from recognition results.

def findKeyWord(list1,list2):
    for word in list1:
        if word in list2: 
            return word
    return None

startTime = time.time()

def debugPrint(t):
    if not DEBUG: return
    print(t)
    # debugFile.write(t)
    # debugFile.write('\n')
    # debugFile.flush()


iconDirectory = os.path.join(status.getUnimacroDirectory(), 'icons')
if not os.path.isdir(iconDirectory):
    raise Exception('icon folder not present (for repeat and waiting icon): {iconDirectory}')

# standard stuff Joel (adapted for possible empty gramSpec, QH, unimacro)
thisGrammar = ThisGrammar()
if thisGrammar.gramSpec:
    thisGrammar.initialize()
else:
    thisGrammar = None

def unload():
    #pylint:disable=W0603
    global thisGrammar
    if thisGrammar: thisGrammar.unload()
    thisGrammar = None

# def changeCallback(type,args):
#     # not active without special version of natlinkmain,
#     # call the cancelMode, to switch off exclusive mode when mic toggles:
#     if ((type == 'mic') and (args=='on')):
#         return   # check WAS in natlinkmain...
#     if thisGrammar:
#         thisGrammar.cancelMode()




"""
This file is part of a Github project (formerly a SourceForge project)
called "unimacro" see http://qh.antenna.nl/unimacro
(c) copyright 2003 see http://qh.antenna.nl/unimacro/aboutunimacro.html
    or the file COPYRIGHT.txt in the natlink\natlink directory 

  _tags.py: make HTML tags
  
  The basics, make a tags work (again), more testing should be done

written by: Quintijn Hoogenboom (QH softwaretraining & advies)
august 2003/March 2022 (python3)/August 2024
#
"""
#pylint:disable=C0116, W0603, W0613, W0201, R0912
import natlink
import unimacro.natlinkutilsbj as natbj
from dtactions.unimacro import unimacroutils
from dtactions.unimacro.unimacroactions import doKeystroke as keystroke
from dtactions.unimacro.unimacroactions import doAction as action
# from dtactions.natlinkclipboard import Clipboard
# natlinkclipboard is not safe at the moment. 

language = unimacroutils.getLanguage()        

ancestor = natbj.IniGrammar
class ThisGrammar(ancestor):
    """grammar that makes html tags, as defined in an inifile
    """

    language = unimacroutils.getLanguage()        
    iniIgnoreGrammarLists = ['character']

    name = "tags"
    gramSpec = """
<tags> exported = (Tag | HTML Tag | <prefix> Tag | <prefix> HTML Tag )({tagname}|{character}+);
<prefix> = Open | Close | Begin | End | Empty;
    """

    def initialize(self):
        if not self.language:
            print("no valid language in grammar "+__name__+" grammar not initialized")
            return

        self.load(self.gramSpec)
        self.setCharactersList('character')        
        self.switchOnOrOff()

    def gotBegin(self,moduleInfo):
        if self.checkForChanges:
            self.checkInifile()

    def gotResultsInit(self,words,fullResults):
        self.letters = ''
        self.pleft = ''
        self.pright = ''
        self.dictated = ''
        self.onlyOpen = self.onlyClose = self.empty = False
        
    def gotResults_tags(self,words,fullResults):
        self.letters = self.getFromInifile(words, 'tagname', noWarning=1)
        if not self.letters:
            print('_tags, no valid tagname found: {words}')
            return
        for w in words:
            char = self.getCharacterFromSpoken(w)
            if char:
                self.letters += char

    #def gotResults_dgndictation(self,words,fullResults):
    #    """return dictated text, put in between (or before or after the tags)
    #    """
    #    self.dictated, dummy = nsformat.formatWords(words, state=-1)  # no capping, no spacing
    #    #print '-result of nsformat: |%s|'% repr(self.dictated)
    

    def gotResults_prefix(self,words,fullResults):
        """set the following instance variables
        """
        self.empty = bool(self.hasCommon(words, 'Empty'))
        self.onlyOpen = bool(self.hasCommon(words, ['Begin', 'Open']))
        self.onlyClose = bool(self.hasCommon(words, ['Close', 'End']))
        

    def gotResults(self,words,fullResults):
        tag = self.letters.strip()
##        print 'rule gotResults: %s'% tag
        pleft = pright = ""
        if not tag:
            return
        pleft = f'<{tag}>'
        if tag.find(' ') >= 0:
            endTag  = ' '.split(tag, maxsplit=1)[0]
        else:
            endTag = tag
        pright = f'</{endTag}>'

        # see of something selected, leave clipboard intact 
        # cb =  Clipboard(save_clear=True)
        unimacroutils.saveClipboard()
        keystroke('{shift+right}')   # take one extra char for the clipboard to hit
        action('<<cut>>')
        action('W')
        cb_text = unimacroutils.getClipboard()
        unimacroutils.restoreClipboard()
        if cb_text:
            contents, lastchar = cb_text[:-1], cb_text[-1]
        else:
            action('<<undo>>')
            raise OSError('no value in clipboard, restore cut text (probably at end of file)')
        print(f'_tags, got from clipboard: "{contents}" + extra char: "{lastchar}"')

        # contents = cb.Get_text()   #.replace('\r','').strip()
        if not self.onlyClose:
            keystroke(pleft)
        if self.empty:
            if contents:
                print(f'_tags: ask for empty, but have contents: "{contents}", ignore these')
        else:
            keystroke(contents)
        if self.onlyOpen:
            if lastchar:
                keystroke(lastchar)
                keystroke('{left %s}'% len(lastchar))
        else:            
            keystroke(pright)
            if lastchar:
                keystroke(lastchar)
                keystroke('{left %s}'% len(lastchar))
            if not self.onlyClose:
                if pright:
                    keystroke('{left %s}'% len(pright))

    def fillDefaultInifile(self, ini):
        """filling entries for default ini file

        """
        ancestor.fillDefaultInifile(self, ini)
        if self.language == 'nld':
            tagNames = {
                'Hedder 1':  'h1',
                'Hedder 2':  'h2',
                'Hedder 3':  'h3',
                'tabel':  'table',
                'tabel honderd procent':  'table border="0" cellpadding="0" cellspacing="0" width="100%"',
                'tabel data':  'td',
                'tabel roo':  'tr',
                'Java script':  'script language="JavaScript"', 
                'script':  'script'
            }
            
        elif self.language == 'enx':
            tagNames = {
                'Header 1':  'h1',
                'Header 2':  'h2',
                'Header 3':  'h3',
                'table':  'table',
                'table data':  'td',
                'table row':  'tr',
                'Java script':  'script language="JavaScript"', 
                'script':  'script'
            }
        else:
            print('-----filling ini file {self.__module__}, invalid language: "{self.language}"!')
            ini.set('general', 'error', 'invalid language')
            return
        for k, v in list(tagNames.items()):
            ini.set('tagname', k, v)
        # by default switch off initially:
        ini.set('general', 'initial on', '0')

            
def stripSpokenForm(w):
    pos = w.find('\\')
    if pos == -1:
        return w
    if pos == 0:
        return ' '
    return w[:pos]

# standard stuff Joel (adapted for possible empty gramSpec, QH, unimacro)
if natlink.isNatSpeakRunning(): 
    thisGrammar = ThisGrammar()
    if thisGrammar.gramSpec:
        thisGrammar.initialize()
    else:
        thisGrammar = None
    
    def unload():
        global thisGrammar
        if thisGrammar:
            thisGrammar.unload()
        thisGrammar = None




#!/usr/bin/env python
#(re)test hotshot output (created with a unittest script)
# in use for unittestMessagefunctions.py (testing test_getting_more_info_from_application)...
import hotshot.stats
filePath = r'U:\messagesfunctionstest.prof'
stats = hotshot.stats.load(filePath)
stats.strip_dirs()
stats.sort_stats('cumulative', 'calls')
stats.print_stats()




#
# Python Macro Language for Dragon NaturallySpeaking
#   (c) Copyright 1999 by Joel Gould
#   Portions (c) Copyright 1999 by Dragon Systems, Inc.
#
# unittestMouse.py
# testing the Unimacro mechanism for mousing, recording a position
# absolute or relative (absorrel),
# with respect to screen, activewindow or client area of active window (0, 1 or 5)
# and with respect to one of the corners of a window/client area (1 or 5)
#
# todo: more monitors.
# Quintijn Hoogenboom, july 2012

import sys
import os
import os.path
import natlink
import math
from dtactions.unimacro.unimacroactions import doAction as action
from dtactions.unimacro import unimacroutils as natqh
print('file: %s'% __file__)
#need this here (hard coded, sorry) for it can be run without NatSpeak being on
extraPaths = r"C:\natlink\unimacro", r"D:\natlink\unimacro"
for extraPath in extraPaths:
    if os.path.isdir(extraPath):
        if extraPath not in sys.path:
            sys.path.append(extraPath)
from dtactions.unimacro import unimacroutils
import sys
import unittest
import unittest
import UnimacroTestHelpers

import os
import os.path
import time
import pprint
import TestCaseWithHelpers
class TestError(Exception):pass

#----------------------------
def getBaseFolder(globalsDict=None):
    """get the folder of the calling module.

    either sys.argv[0] (when run direct) or
    __file__, which can be empty. In that case take the working directory
    """
    globalsDictHere = globalsDict or globals()
    baseFolder = ""
    if globalsDictHere['__name__']  == "__main__":
        baseFolder = os.path.split(sys.argv[0])[0]
        print('baseFolder from argv: %s'% baseFolder)
    elif globalsDictHere['__file__']:
        baseFolder = os.path.split(globalsDictHere['__file__'])[0]
        print('baseFolder from __file__: %s'% baseFolder)
    if not baseFolder or baseFolder == '.':
        baseFolder = os.getcwd()
        print('baseFolder was empty, take wd: %s'% baseFolder)
    return baseFolder

thisDir = getBaseFolder(globals())

natconnectOption = 0 # or 1 for threading, 0 for not. Seems to make difference
                     # with spurious error (if set to 1), missing gotBegin and all that...
logFileName = os.path.join(thisDir, "testresult.txt")



#---------------------------------------------------------------------------
class UnittestMouse(TestCaseWithHelpers.TestCaseWithHelpers):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass

    def tttest_getRectData(self):
        """test the small function getRectData in natlinkutilsqh
        
        give, from a given rectangle, width, height, xMin, yMin, xMax, yMax
        """
        ##     xMin, yMin, xMax, yMax
        rect = 10, 120, 1000, 900
        ##   width, heigth,  xMin, yMin, xMax, yMax
        expected = (990, 780, 10, 120, 1000, 900)
        got = natlinkutilsqh.getRectData(rect)
        self.assert_equal(expected, got, 'getRectData of "%s" did not return the correct results'% repr(rect))

        # rect can be tuple or list:
        ##     xMin, yMin, xMax, yMax
        rect = [-1000, -20, 0, 300]
        ##   width, heigth,  xMin, yMin, xMax, yMax
        expected =  (1000, 320, -1000, -20, 0, 300)
        got = natlinkutilsqh.getRectData(rect)
        self.assert_equal(expected, got, 'getRectData of "%s" did not return the correct results'% rect)

    def tttest_coordToRel_and_relToCoord(self):
        """test the functions to calculate relative coordinates and back
        
        functions are in natlinkutilsqh.py and are used with doMouse and getMousePosition
        
        (x can be exchanged to y of course)
        """
        xMin, xMax = 20, 1000
        # from xMin (relX positive)
        for x in (20, 200, 500, 800, 900, 990, 999):
            relX = natlinkutilsqh.coordToRel(x, xMin, xMax)
            self.assertTrue(relX >=0, "coordToRel, result should be positive (x: %s, xMin, %s, xMax: %s, relX: %.6s"%
                         (x, xMin, xMax, relX))
            relX = round(relX, 3)
            result = natlinkutilsqh.relToCoord(relX, xMin, xMax)
            self.assert_equal(x, result, "relative value changes coordinate: %s to %s"% (x, result), epsilon=0)

        # from xMax (relX negative)
        for x in (20, 200, 500, 800, 900, 990, 999):
            relX = natlinkutilsqh.coordToRel(x, xMin, xMax, side=1)
            relX = round(relX, 3)
            self.assertTrue(relX < 0, "coordToRel, result should be negative (x: %s, xMin, %s, xMax: %s, relX: %.6s"%
                         (x, xMin, xMax, relX))
            result = natlinkutilsqh.relToCoord(relX, xMin, xMax)
            self.assert_equal(x, result, "relative value changes coordinate (from xMax): %s to %s (relX: %.6s"% (x, result, relX), epsilon=0)


    def tttest_getOutsideWindow(self):
        """try to position on the taskbar and get the position
        """
        
        action("RMP(5, 1, 0.5, noclick)") # relative in foreground window, tune to your testing!
        action("MP(2, 500, 0, noclick)")   # move a bit to the right
        initialPosition = natlink.getCursorPos()
        mousePos = natlinkutilsqh.getMousePosition() # abs, screen, corner not relevant
        if mousePos is None:
            self.fail("could not get mouse position")
        x, y = mousePos
        absorrel, which = 0, 0
        natlinkutilsqh.doMouse(absorrel, which, x, y, mouse="noclick")
        finalPosition = natlink.getCursorPos()
        log('command outside window(?): %s, %s, %s, %s, initial: %s, final: %s'% (absorrel, which, x, y, repr(initialPosition), repr(finalPosition)))
        self.assert_equal(initialPosition, finalPosition, "position of mouse after move outside the current window should be the same\n"
                "absorrel: %s, which: %s"% (absorrel, which))


        
        
    def tttest_getPositions(self):
        """test setting and getting positions in all possible ways
        
        use action RMP to go to the wanted position.
        
        then each time get a position
        (absolute/relative, absorrel, screen, active window or client area (0, 1, 5),
        and relative to one of the corners (0, 1, 2, 3))
        setting it again and see if same position is found
        
        get with: getMousePosition
        set with: doMouse
        """
        #absorrel = 0 # absolute
        #which = 0 # whole screen
        #corner = 0 # top left
        action("RMP(5, 0.3, 0.4, noclick)") # relative in foreground window, tune to your testing!
        
        initialPosition = natlink.getCursorPos()
        for absorrel in (0, 1):
            for which in (0, 1, 5):
                for cornerPos in range(4):
                    #absorrel, which, cornerPos = 1, 0, 0
                    #absorrel = 1
                    # each test, little shifting for relative allowed
                    if absorrel:
                        epsilon = 0 # relative, position may shift a bit...
                    else:
                        epsilon = 0  # absolute, caculations must fit
                    if absorrel == 0 and which == 0 and cornerPos > 0:
                        print('invalid combination to test: absorrel: %s, which: %s, cornerPos: %s'% (absorrel, which, cornerPos))
                        continue
                    initialPosition = natlink.getCursorPos()
                        
                    mousePos = natlinkutilsqh.getMousePosition(absorrel, which, cornerPos)
                    if mousePos is None:
                        self.fail('getMousePosition in test suite should not result in None (absorrel: %s, which: %s, cornerPos: %s)'%
                                   (absorrel, which, cornerPos))
                    x, y = mousePos
                    natlinkutilsqh.doMouse(absorrel, which, x, y, mouse="noclick")
                    finalPosition = natlink.getCursorPos()
                    log('command: %s, %s, %s, %s, initial: %s, final: %s'% (absorrel, which, x, y, repr(initialPosition), repr(finalPosition)))
                    self.assert_equal(initialPosition, finalPosition, "position of mouse after simple action should be the same\n"
                            "absorrel: %s, which: %s, cornerPos: %s"% (absorrel, which, cornerPos),
                            epsilon=epsilon)
  
    def test_doMouse(self):
        """problem with doMouse (natlinkutilsqh) with second screen LEFT of first"""
        unimacroutils.doMouse(0, 0, 35, 188, 0)
        pass
                            

def log(t):
    """log to print and file if present

    note print depends on the state of natlink: where is goes or disappears...
    I have no complete insight is this, but checking the logfile afterwards
    always works (QH)
    """
    print(t)
    if logFile:
        logFile.write(t + '\n')


logFile = None

def dumpResult(testResult, logFile):
    """dump into 
    """
    if testResult.wasSuccessful():
        mes = "all succesful"
        logFile.write(mes)
        return
    logFile.write('\n--------------- errors -----------------\n')
    for case, tb in testResult.errors:
        logFile.write('\n---------- %s --------\n'% case)
        logFile.write(tb)
        
    logFile.write('\n--------------- failures -----------------\n')
    for case, tb in testResult.failures:
        logFile.write('\n---------- %s --------\n'% case)
        logFile.write(tb)

def run():
    global logFile, natconnectOption
    logFile = open(logFileName, "w")
    log("log messages to file: %s"% logFileName)
    log('starting UnittestMouse')
    # trick: if you only want one or two tests to perform, change
    # the test names to her example def tttest....
    # and change the word 'test' into 'tttest'...
    # do not forget to change back and do all the tests when you are done.
    suite = unittest.makeSuite(UnittestMouse, 'test')
##    natconnectOption = 0 # no threading has most chances to pass...
    log('\nstarting tests with threading: %s\n'% natconnectOption)
    natlink.natConnect(natconnectOption)
    result = unittest.TextTestRunner().run(suite)
    dumpResult(result, logFile=logFile)
    natlink.natDisconnect()
    logFile.close()

    

if __name__ == "__main__":
    run()



#
# Python Macro Language for Dragon NaturallySpeaking
#   (c) Copyright 1999 by Joel Gould
#   Portions (c) Copyright 1999 by Dragon Systems, Inc.
#
# unittestFolders.py
#   This script performs some testing of the folders grammar
#   the site mechanism (of Qh private) in which modules of a website generating program
#   are tested specifically
#
import sys
import unittest
import types
import os
import os.path
import TestCaseWithHelpers
import natlink


def getBaseFolder(globalsDict=None):
    """get the folder of the calling module.

    either sys.argv[0] (when run direct) or
    __file__, which can be empty. In that case take the working directory
    """
    globalsDictHere = globalsDict or globals()
    baseFolder = ""
    if globalsDictHere['__name__']  == "__main__":
        baseFolder = os.path.split(sys.argv[0])[0]
        print('baseFolder from argv: %s'% baseFolder)
    elif globalsDictHere['__file__']:
        baseFolder = os.path.split(globalsDictHere['__file__'])[0]
        print('baseFolder from __file__: %s'% baseFolder)
    if not baseFolder or baseFolder == '.':
        baseFolder = os.getcwd()
        print('baseFolder was empty, take wd: %s'% baseFolder)
    return baseFolder

thisDir = getBaseFolder(globals())
logFileName = os.path.join(thisDir, "testresult.txt")

#---------------------------------------------------------------------------
# These tests should be run after we call natConnect
# no reopen user at each test anymore..
# no default open window (open window will be the calling program)
# default .ini files pop up when you first run this test. just ignore them.
class UnittestFolders(TestCaseWithHelpers.TestCaseWithHelpers):
    def setUp(self):
        natlink.natConnect()
        self.folders = __import__("_folders")
        self.grammarInstance = self.folders.thisGrammar
    def tearDown(self):
        self.folders.unload()
        natlink.natDisconnect()



    def tttestConnectingWithSite(self):
        """test the connecting of a siteinstance
        
        a way to get in with the debugger of Komodo:
        """
        func = self.grammarInstance.getSiteInstance
        print('func: %s'% func)
        result = func('sg')
        print('result: %s'% result)

    def tttestGetActiveExplorer(self):
        """test the connecting of an active explorer
        
        a way to get in with the debugger of Komodo:
        """
        func = self.grammarInstance.getActiveFolder
        # fill here the windows hndle... (give window info from _general)
        hndle = 788422  # #32770, 
        hndle = 265916  # explorer, 
        print('func: %s'% func)
        result = func(hndle)
        print('result: %s'% result)
    
    def tttestGet_current_directory(self):
        """test getting the current directory
        
        This one not really needed, better go with messages functions, use self.activeFolder from gotBegin.
        
        a way to get in with the debugger of Komodo:
        """
        hndle = 265916  # explorer, 
        func = self.grammarInstance.get_current_directory
        # fill here the windows hndle... (give window info from _general)
        result = func(hndle)
        print('result: %s'% result)

    def testToPartOfRecognition(self):
        """test getting hasCommon function in remember rule
        
        
        a way to get in with the debugger of Komodo:
        """
        # vertalingen is moeilijk, language = zyx.
        func = self.grammarInstance.gotResults_remember
        self.grammarInstance.catchRemember = 'folder'
        self.grammarInstance.nextRule = "dgndictation"
        # fill here the windows hndle... (give window info from _general)
        result = func(['deze', 'folder', 'onthoud', 'als'], {})
        print('result: %s'% result)
    
    
            
def log(t):
    print(t)

def run():
    log('starting unittestFolders')
    # trick: if you only want one or two tests to perform, change
    # the test names to her example def test....
    # and change the word 'test' into 'tttest'...
    # do not forget to change back and do all the tests when you are done.
    suite = unittest.makeSuite(UnittestFolders, 'test')
##    natconnectOption = 0 # no threading has most chances to pass...
    result = unittest.TextTestRunner().run(suite)
    print(result)
if __name__ == "__main__":
    run()



# (unimacro - natlink macro wrapper/extensions)
# (c) copyright 2003 Quintijn Hoogenboom (quintijn@users.sourceforge.net)
#                    Ben Staniford (ben_staniford@users.sourceforge.net)
#                    Bart Jan van Os (bjvo@users.sourceforge.net)
#
# This file is part of a SourceForge project called "unimacro" see
# http://unimacro.SourceForge.net).
#
# "unimacro" is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, see:
# http://www.gnu.org/licenses/gpl.txt
#
# "unimacro" is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; See the GNU General Public License details.
#
# "unimacro" makes use of another SourceForge project "natlink",
# which has the following copyright notice:
#
# Python Macro Language for Dragon NaturallySpeaking
#   (c) Copyright 1999 by Joel Gould
#   Portions (c) Copyright 1999 by Dragon Systems, Inc.
#
# solitaire.py
#
# Written by: Quintijn Hoogenboom (QH softwaretraining &
# advies),2002, revised October 2003
# march 2011: change sol to solitaire
#pylint:disable=C0209
#pylint:disable=E0602

"""Grammar for playing solitaire (patience) hands-free

Extensive use is made of mouse (dragging) routines.
"""

import win32gui
from natlinkcore import natlinktimer
from dtactions.unimacro import unimacroutils
import unimacro.natlinkutilsbj as natbj

#aantal carden en aantal stapelplaatsen:
cnum = list(range(1,8))
snum = list(range(1,5))
#card sizes and spacing:
#kwidth, kheight = 72, 96
kwidth, kheight = 93, 121
hspace, vspace = 10,6
cardhspace, cardvspace = 5, 6 # space in stack (horizontal) and on stacks (vertical)
# for mouse lower or higher:
deltaY = 14

# waiting times:
pauzesDelta = 0.1
minPause = 0.5

ancestor = natbj.DocstringGrammar
class ThisGrammar(ancestor):
    name = 'solitaire'
    iniIgnoreGrammarLists = ['snum', 'cnum'] # are set in this module

    #gramSpec = """ # in the rules docstrings!
    #    """

    def initialize(self):
        if self.load(self.gramSpec):
            self.setNumbersList("cnum", cnum)
            self.setNumbersList("snum", snum)
            print('grammar solitaire (%s) active'% self.name)
            self.prevHandle = -1
            self.stackAuto = 0
            self.pause = 1
            self.inTimer = 0
            self.pauseTime = 4
                        
    def cancelMode(self):
        #self.setExclusive(0)
        if self.inTimer:
            print('cancel timer')
            natlinktimer.setTimerCallback(self.onTimer,0)
            self.inTimer = 0

    def onTimer(self):
        if natlink.getMicState() != 'on':
            print('mic switched off, cancel timer')
            self.cancelMode()
            return
        modInfo = natlink.getCurrentModule()
        if modInfo[2] != self.prevHandle:
            print('window handle changed, cancel timer')
            self.cancelMode()
            return
        # print('in onTimer: %.1f'% time.clock())
        self.rule_newcard([])
        

    def gotBegin(self,moduleInfo):
        print('gotbegin, cancelmode')
        self.cancelMode()
        winHandle = moduleInfo[2]
        if self.prevHandle == winHandle:
            return
        self.prevHandle = winHandle
        if moduleInfo[0].lower().find('solitaire.exe') > 0 and unimacroutils.isTopWindow(moduleInfo[2]):
            if self.checkForChanges:
                print('grammar solitaire (%s) checking the inifile'% self.name)
                self.checkInifile()
            print('self.mayBeSwitchedOn: %s'% self.mayBeSwitchedOn)
            exclusive = self.mayBeSwitchedOn == 'exclusive'
            self.activateAll(exclusive=exclusive)
        else:
            self.deactivateAll()

    #  Bij het initialiseren wordt de horizontale spatiejering uitgerekend.
    #    Er wordt van uitgegaan dat het venster niet te smal staat.
    def gotResultsInit(self,words,fullResults):
        """initialize screen sizes (guesses)
        """
        global hspace, vspace, cardhspace, cardvspace
        clrect = win32gui.GetClientRect(self.prevHandle)
        xMax, _yMax = clrect[2]-1, clrect[3]-1
        hspace = (xMax - 7*kwidth)/8
        cardhspace = int(hspace/2)
        cardvspace = vspace
        ## print 'hSpace: %s, xMax: %s, yMax: %s'% (hspace, xMax, yMax)

    def rule_auto(self, words):
        "stack automatically"
        self.stackAuto = 1

    def rule_exclusive(self, words):
        "solitaire [non] exclusive"
        #switches on auto stacking, {ctrl+a} after each move
        self.prevHandle = None # to reactivate in gotBegin
        self.cancelMode()
        if self.hasCommon(words, 'non'):
            self.mayBeSwitchedOn = 1
        else:
            self.mayBeSwitchedOn = 'exclusive'

    # say stop, or even undo the last action
    def rule_stop_undo(self, words):
        "'hold on'|stop|undo"
        self.cancelMode()
        if self.hasCommon(words, 'undo'):
            keystroke('{ctrl+z}')

    #  Go to a card and click on it
    def rule_card(self, words):
        "card {cnum}"
        k = self.getNumberFromSpoken(words[-1])
        self.moveTo(cardpos(k))
        unimacroutils.buttonClick()

    #  Draw a new card and position on the last opened card
    def rule_newcard(self, words):
        "'new card'|next|continue"
        self.moveTo(firstrowpos(1))
        unimacroutils.buttonClick()
        self.moveTo(firstrowpos(2))
        if self.hasCommon(words, 'continue'):
            timeEachMilliseconds = max(1, self.pauseTime)*500
            print('set the timer to %s'% timeEachMilliseconds)
            natlinktimer.setTimerCallback(self.onTimer, timeEachMilliseconds)
            self.inTimer = 1
            
    # Sometimes the mouse is a bit too high (if the stack grows longer)
    # with this command you position the mouse a bit higher or lower
    def rule_lower(self,words):
        "mouse (lower|higher) [{cnum}]"
        n = self.getNumberFromSpoken(words[-1]) or 1
        d = n*deltaY
        if self.hasCommon(words, 'lower'):
            pass
        elif self.hasCommon(words, 'higher'):
            d = -d
        unimacroutils.doMouse(0,2,0,d,0)

    #  Deze grammatica regel stelt de waittijd na elk (deel) van een
    #    commando in. Zie "pauzes" bovenin dit standpunt
    def rule_wait(self, words):
        "pauses (0 |{cnum})"
        i = int(words[-1])
        self.pause = i

    #  Drag a card from the current position to one of the card columns or
    #                                        to one of the 4 stacks
    def rule_to(self, words):
        "to ((stack {snum})|{cnum})"
        
        to = self.getNumberFromSpoken(words[-1])
        unimacroutils.rememberMouse()

        if self.hasCommon(words, 'stack'):
            to = firstrowpos(to+3)
        else:
            to = cardpos(to)
        self.dragTo(to)
        unimacroutils.cancelMouse()
        unimacroutils.buttonClick()

    def rule_testposition(self, words):
        "test position ((stack {snum})|{cnum})"
        #test the stack and piles positions
        #lefttop = (0,0)
        #unimacroutils.doMouse(0, 5, 0,0, 'move')
        #x, y = unimacroutils.getMousePosition(1)
        #print 'screen x,y: %s, %s'% (x,y)
        #x, y = unimacroutils.getMousePosition(5)
        #print 'client: x,y: %s, %s'% (x,y)
        #time.sleep(2)
        to = self.getNumberFromSpoken(words[-1])
        unimacroutils.rememberMouse()

        if self.hasCommon(words, 'stack'):
            to = firstrowpos(to+3)
        else:
            to = cardpos(to)
        print('position: %s'% repr(to))
        self.moveTo(to)
        
    def rule_cardnumto(self, words):
        "[card] {cnum} to ((stack {snum})|{cnum})"
        #card with number to a stack or another card column
        #
        #the first word is optional,and is recognised with the function
        #self.hasCommon, which can handle translations or synonyms
        print('cardnumto: %s'% words)
        unimacroutils.rememberMouse()
        # if self.hasCommon(words[0],['card']):
        #     ww = words[1:]
        # else:
        #     ww = words[:]

        # go to numbered card with mouse:
        From = self.getNumberFromSpoken(words[0])
        self.moveTo(cardpos(From))        

        unimacroutils.rememberMouse()
        to = self.getNumberFromSpoken(words[-1])

        # check if you go to a stack or another card column:        
        if self.hasCommon(words, 'stack'):
            to = firstrowpos(to+3)
        else:
            to = cardpos(to)
        self.dragTo(to)
        unimacroutils.cancelMouse()
        unimacroutils.buttonClick()
    
    def rule_cardto(self, words):
        "card to ((stack {snum})|{cnum})"
        #drag the last drawn card to a stack or to a pile
        print('cardto: %s'% words)

        unimacroutils.rememberMouse()
        self.moveTo(firstrowpos(2))        
        to = self.getNumberFromSpoken(words[-1])
        
        if self.hasCommon(words, 'stack'):
            to = firstrowpos(to+3)
        else:
            to = cardpos(to)
        self.dragTo(to)
        unimacroutils.cancelMouse()
        unimacroutils.buttonClick()
    
    def gotResults(self,words,fullResults):
        """if stack auto, do after each move a {ctrl+a}"""
        if self.stackAuto:
            keystroke('{ctrl+a}')
        
    #  paused a given number of milliseconds 
    #   if this variable = 0 no pausing is done.        
    #    The length of the pauses can be given with
    #    command "pauzes (0 | 1 | 2 | 3 | 4 | 5 | 6 | 7)". 
    #   At the top of this file the increment of the pausing is given.
    def Wait(self, t=None):
        if not t:
            unimacroutils.Wait(self.pause*pauzesDelta+minPause)
        else:
            unimacroutils.Wait(t)

    # move to the given position and have a short wait:
    def moveTo(self, pos):
        unimacroutils.doMouse(0,5, pos[0], pos[1], 'move')
        self.Wait()

    # drag from current position to the new position
    # Pause a few times, dependent on the pause state.
    def dragTo(self, pos):
        xold,yold = unimacroutils.getMousePosition(5)
        print('hold down: %s, %s'% (xold, yold))
        unimacroutils.doMouse(0,5,xold, yold, 'down')
        xyincr = 50
        nstepsx = int(abs(pos[0]-xold)/xyincr)
        nstepsy = int(abs(pos[1]-yold)/xyincr)
        nsteps = max(nstepsx, nstepsy)
        print('nstepsx: %s, nstepsy: %s, nsteps: %s'% (nstepsx, nstepsy, nsteps))
        ysteps = int((pos[1]-yold)/nsteps)
        xsteps = int((pos[0]-xold)/nsteps)
        x, y = xold, yold
        for _ in range(nsteps):
            x += xsteps
            y += ysteps
            unimacroutils.doMouse(0,5, x, y, 'move')
            self.Wait(0.01)
        if x != pos[0] or y != pos[1]:
            print('final move: %s, %s, %s, %s'% (x, pos[0], y, pos[1]))
            unimacroutils.doMouse(0,5, pos[0], pos[1], 'move')
            self.Wait(0.01)            
        unimacroutils.doMouse(0,5, pos[0], pos[1], 'move')
        self.Wait(0.01)
        unimacroutils.releaseMouse()
        self.Wait()

            
#  Het berekenen van de coordinaten van cardnummer,i kan liggen
#    tussen 1 en 
def cardpos(i):
    x = i*(hspace+kwidth)-0.5*kwidth
    y = int(2*vspace) + int(2.1*kheight)
    return x,y

# Het berekenen van een positie van een card op de eerste rij.
# Positie 1 is de stapel waaruit gedeeld wordt,
# positie 2 is het stapeltje gedeelde carden,
# en positie i + 3 is de positie van een stapel (i tussen 1 en 4)
def firstrowpos(i):
    """calculate the positions of the first row
    
    1 is pile of closed cards (to click on for new card)
    2 is last cards drawn
    i+3 is stack (stapel) 1,2,3,4
    """
    x = i*(hspace+kwidth)-0.5*kwidth
    if i == 2:
        x += cardhspace
    y = vspace + cardvspace + int(0.5*kheight)
    return x,y

# standard stuff Joel (adapted for possible empty gramSpec, QH, unimacro)
thisGrammar = ThisGrammar()
if thisGrammar.gramSpec:
    thisGrammar.initialize()
else:
    thisGrammar = None

def unload():
    #pylint:disable=W0603
    global thisGrammar
    if thisGrammar:
        thisGrammar.unload()
    thisGrammar = None

# def changeCallback(type,args):
#     # not active without special version of natlinkmain,
#     # call the cancelMode, to switch off exclusive mode when mic toggles:
#     if ((type == 'mic') and (args=='on')):
#         return   # check WAS in natlinkmain...
#     if thisGrammar:
#         print('cancel')
#         thisGrammar.cancelMode()



# Python Macro Language for Dragon NaturallySpeaking
#   (c) Copyright 1999 by Joel Gould
#   Portions (c) Copyright 1999 by Dragon Systems, Inc.
#
# unittestActions.py
#   This script performs some basic tests of the NatLink but
#   with the Actions subclass.
#   especially the numbers parts of a spoken form list
#
# run from a (preferably clean) US user profile, easiest from IDLE.
# do not run from pythonwin. 
#
import sys
import unittest
import os
import os.path
import time
from pathlib import Path
import natlink
from natlinkcore import natlinkstatus
from dtactions.unimacro.unimacroactions import doAction as action
from dtactions.unimacro.unimacroactions import doKeystroke as keystroke
from dtactions.unimacro import unimacroactions as actions
import TestCaseWithHelpers

status = natlinkstatus.NatlinkStatus()

class TestError(Exception):
    pass
ExitQuietly = 'ExitQuietly'

def getBaseFolder(globalsDict=None):
    """get the folder of the calling module.

    either sys.argv[0] (when run direct) or
    __file__, which can be empty. In that case take the working directory
    """
    globalsDictHere = globalsDict or globals()
    baseFolder = ""
    if globalsDictHere['__name__']  == "__main__":
        baseFolder = os.path.split(sys.argv[0])[0]
        print('baseFolder from argv: %s'% baseFolder)
    elif globalsDictHere['__file__']:
        baseFolder = os.path.split(globalsDictHere['__file__'])[0]
        print('baseFolder from __file__: %s'% baseFolder)
    if not baseFolder or baseFolder == '.':
        baseFolder = os.getcwd()
        print('baseFolder was empty, take wd: %s'% baseFolder)
    return baseFolder

thisDir = Path(getBaseFolder(globals()))

natconnectOption = 0 # or 1 for threading, 0 for not. Seems to make difference
                     # with spurious error (if set to 1), missing gotBegin and all that...
logFileName = os.path.join(thisDir, "testresult.txt")

testFilesDir = Path(thisDir)/'test_clipboardfiles'
if testFilesDir.is_dir():
    print("test files for Bringup: %s"% testFilesDir)
else:
    raise OSError("no valid directory for test files: %s"% testFilesDir)
#---------------------------------------------------------------------------
# These tests should be run after we call natConnect
# no reopen user at each test anymore..
# no default open window (open window will be the calling program)
# default .ini files pop up when you first run this test. just ignore them.
class UnittestActions(TestCaseWithHelpers.TestCaseWithHelpers):
    def setUp(self):
        # if not natlink.isNatSpeakRunning():
        #     raise TestError('NatSpeak is not currently running')
        # self.connect()
        # self.user = natlink.getCurrentUser()[0]
        # self.setMicState = "off"
        pass

    def tearDown(self):
        # try:
        #     # give message:
        #     self.setMicState = "off"
        #     # kill things
        #     self.clearTestFiles()
        # finally:
        #     self.disconnect()
        pass
        
    def connect(self):
        # start with 1 for thread safety when run from pythonwin:
        natlink.natConnect(natconnectOption)

    def disconnect(self):
        natlink.natDisconnect()
        
    def log(self, t):
        # only log to file:
        log(t)

    def clearTestFiles(self):
        """remove .py and .pyc files from the natlinkmain test

        """
        return
        # try to remove __main__.ini, but gives unexpected windows popping up
        #userdir = status.getUnimacroUserDirectory()
        #language = status.getLanguage()
        #print 'userdir: %s, language: %s'% (userdir, language)
        #userinidir = os.path.join(userdir, '%s_inifiles'%language)
        #if os.path.isdir(userinidir):
        #    mainfile = os.path.join(userinidir, '__main__.ini')
        #    if os.path.isfile(mainfile):
        #        os.remove(mainfile)
        #else:
        #    raise OSError("clearTestFiles, should be a valid directory: %s"% userinidir)

    def wait(self, t=1):
        time.sleep(t)


    #---------------------------------------------------------------------------
    # This utility subroutine executes a Python command and makes sure that
    # an exception (of the expected type) is raised.  Otherwise a TestError
    # exception is raised

    def doTestForException(self, exceptionType,command,localVars={}):
        try:
            exec(command,globals(),localVars)
        except exceptionType:
            return
        raise TestError('Expecting an exception to be raised calling '+command)

    def testKeystroke(self):
        """test in foreground window do some keystrokes with marks functions
        
        """
        log("test keystroke in the foreground window")
        # this one works, no ini file needed for the "hard key"
        keystroke("aaa{left 3}{del 3}")
        # keystroke("{shift+home}", hardKeys="{home}")
        # keystroke("{shift+home}", hardKeys=["{home}"])
        keystroke("xyz{shift+left 3}{ctrl+x}")
        

        
        # now get keystrokes AS IF in default child window:
        # proginfo = 
        
        
        # 


    def tttestGetSectionList(self):
        """test with fake program info for sections to be selected
        """
        progInfo = (r'C:\Windows\SysWOW64\notepad.exe', 'notepad', 'Untitled - Notepad', 'child', 'Notepad', 12345)
        sectionsList = actions.getSectionList(progInfo)
        expList = []
        
        

    def testAutoHotKey(self):
        """test ahk scripts
        """
        #action("AHK showmessageswindow.ahk")
    
        # action("AHK runcontrolget.ahk")
        
        action("AHK send hello")  ###h
        action("AHK send {backspace 4}")
        #

    def testUnimacroBringup(self):
        """test ahk scripts
        """
        notepadFile = testFilesDir/"testempty.txt"
        action("BRINGUP %s"% notepadFile)
        
#action("AHK send hello")
        

def log(t):
    """log to print and file if present

    note print depends on the state of natlink: where is goes or disappears...
    I have no complete insight is this, but checking the logfile afterwards
    always works (QH)
    """
    print(t)
    if logFile:
        logFile.write(t + '\n')
    
#---------------------------------------------------------------------------
# run
#
# This is the main entry point.  It will connect to NatSpeak and perform
# a series of tests.  In the case of an error, it will cleanly disconnect
# from NatSpeak and print the exception information,
def dumpResult(testResult, logFile):
    """dump into 
    """
    if testResult.wasSuccessful():
        mes = "all succesful"
        logFile.write(mes)
        return
    logFile.write('\n--------------- errors -----------------\n')
    for case, tb in testResult.errors:
        logFile.write('\n---------- %s --------\n'% case)
        logFile.write(tb)
        
    logFile.write('\n--------------- failures -----------------\n')
    for case, tb in testResult.failures:
        logFile.write('\n---------- %s --------\n'% case)
        logFile.write(tb)

    


logFile = None

def run():
    global logFile, natconnectOption
    logFile = open(logFileName, "w")
    log("log messages to file: %s"% logFileName)
    log('starting unittestNatlink')
    # trick: if you only want one or two tests to perform, change
    # the test names to her example def test....
    # and change the word 'test' into 'tttest'...
    # do not forget to change back and do all the tests when you are done.
    suite = unittest.makeSuite(UnittestActions, 'test')
##    natconnectOption = 0 # no threading has most chances to pass...
    log('\nstarting tests with threading: %s\n'% natconnectOption)
    result = unittest.TextTestRunner().run(suite)
    dumpResult(result, logFile=logFile)
    
    logFile.close()

if __name__ == "__main__":
    run()



#Bart Jan's work, needs testing presently
#pylint:disable = W0612, W0702, C0321, R0913, C0209
import time
import pickle   

from pywin.mfc import dialog
from pywin.framework import dlgappcore, app
import win32ui
import win32con
import commctrl
import win32api
from dtactions.unimacro.unimacroutils import AppBringUp

from natlinkutilsbj import *
status = natlinkstatus.NatlinkStatus()
dataDirectory = status.getUnimacroDataDirectory()
# print(f'dataDirectory: {dataDirectory}')


IDC_NUMBERS=1010

class ListDialog (dialog.Dialog):  # Taken from the demo; adapted

    def __init__ (self, title, List, heading='item',size=(300,200),okButton='OK',resize=None):
        self.iconId = win32ui.IDR_MAINFRAME
##        print 'title: %s'% title
##        print 'List: %s'% List
        id = self._maketemplate(title, size, okButton, resize)
        print('id: %s'% id)
        dialog.Dialog.__init__ (self, id)
        self.Owner=None
        self.defer=None
        self.HookMessage (self.on_size, win32con.WM_SIZE)
        self.HookNotify(self.OnListItemChange, commctrl.LVN_ITEMCHANGED)
        self.HookCommand(self.OnListClick, win32ui.IDC_LIST1)
        self.heading=heading
        self.items = List
        self.size=size

    def _maketemplate(self, title, size, okButton="OK", resize=None):
        style = win32con.WS_DLGFRAME | win32con.WS_SYSMENU | win32con.WS_VISIBLE
        if resize:
            style=style | win32con.WS_MAXIMIZEBOX | win32con.WS_MINIMIZEBOX | win32con.WS_SIZEBOX
        ls = (
            win32con.WS_CHILD           |
            win32con.WS_VISIBLE         |
            commctrl.LVS_ALIGNLEFT      |
            commctrl.LVS_REPORT
            )
        bs = (
            win32con.WS_CHILD           |
            win32con.WS_VISIBLE
             )
        ns = (
            win32con.WS_CHILD           |
            win32con.WS_VISIBLE         |  
            commctrl.LVS_ALIGNLEFT      |
            commctrl.LVS_NOCOLUMNHEADER |
            commctrl.LVS_NOSCROLL |
            commctrl.LVS_REPORT
            )
        return [ [title, (0, 0, 200, 200), style, None, (8, "MS Sans Serif")],
            ["SysListView32", None, win32ui.IDC_LIST1, (0, 0, 200, 200), ls], 
            [128,   "OK", win32con.IDOK, (10, 0, 50, 14), bs | win32con.BS_DEFPUSHBUTTON],
            [128,   "Cancel",win32con.IDCANCEL,(100, 0, 50, 14), bs],
            ["SysListView32", None, IDC_NUMBERS, (0, 0, 450, 200), ns],                 
            ]
    

    def FillList(self):
        size = self.GetWindowRect()
        width = size[2] - size[0] - (10)
        itemDetails = (commctrl.LVCFMT_LEFT, width, self.heading, 0)
        self.itemsControl.InsertColumn(0, itemDetails)
        index = 0
        for item in self.items:
            #preceding spaces are stripped to facilitate list completion
            itemText=str(item).lstrip()
            index = self.itemsControl.InsertItem(index+1, itemText, 0)

    def OnListClick(self, id, code):
        if code==commctrl.NM_DBLCLK:
            self.EndDialog(win32con.IDOK)
        return 1

    def OnListItemChange(self,std, extra):
        (hwndFrom, idFrom, code), (itemNotify, sub, newState, oldState, change, point, lparam) = std, extra
        oldSel = (oldState & commctrl.LVIS_SELECTED)!=0
        newSel = (newState & commctrl.LVIS_SELECTED)!=0
        if oldSel != newSel:
            try:
                if newSel:
                    self.SelItems.append(itemNotify)
                else:
                    if itemNotify in self.SelItems:
                        self.SelItems.remove(itemNotify)
                self.butOK.EnableWindow(1)
            except win32ui.error:
                self.SelItems=[]

    def OnInitDialog (self):
        rc = dialog.Dialog.OnInitDialog (self)
        self.itemsControl = self.GetDlgItem(win32ui.IDC_LIST1)
        self.butOK = self.GetDlgItem(win32con.IDOK)
        self.butCancel = self.GetDlgItem(win32con.IDCANCEL)
        self.numbersControl = self.GetDlgItem(IDC_NUMBERS)
        self.numbersControl.EnableWindow(0)
        w=self.numbersControl.GetStringWidth('22')+6
        itemDetails = (commctrl.LVCFMT_LEFT, w, '', 0)
        self.numbersControl.InsertColumn(0, itemDetails)

        self.SelItems=[]
        self.MoveWindow((0,0,self.size[0],self.size[1]))
        self.CenterWindow()
        size = self.GetWindowRect()
        self.LayoutControls(size[2]-size[0], size[3]-size[1])
                 
        self.FillList()
        #Select First Item
        self.itemsControl.SetItemState(0,commctrl.LVNI_SELECTED+commctrl.LVNI_FOCUSED,255)

        #ShowWindow(win32con.SW_RESTORE)
        try:
            parent=win32ui.GetMainFrame()
        except:
            parent=None
        if parent: parent.ShowWindow(win32con.SW_RESTORE)
        return rc

    def ItemHeight(self):
        try:
            (x0,y0,x1,y1)=self.itemsControl.GetItemRect(1,0)
            return 3+y1-y0
        except:
            return 20

    def NItemsVisible(self):
        itemsLeft=self.itemsControl.GetItemCount()-self.itemsControl.GetTopIndex()
        return min(self.itemsControl.GetCountPerPage(),itemsLeft)
        
    def LayoutControls(self, w, h):
        nw=self.numbersControl.GetColumnWidth(0)
        self.itemsControl.MoveWindow((4+nw,15,w-2,h-30))
        self.numbersControl.MoveWindow((2,15+self.ItemHeight(),2+nw,h-(30)))
        NItems=self.NItemsVisible()
        self.numbersControl.DeleteAllItems()        
        for i in range(1,NItems+1):
            self.numbersControl.InsertItem(i, '%2i'%i, 0)
        self.butCancel.MoveWindow((w-60, h-24, w-10, h-4))
        self.butOK.MoveWindow((w-120, h-24, w-70, h-4))

    def on_size (self, params):
        lparam = params[3]
        w = win32api.LOWORD(lparam)
        h = win32api.HIWORD(lparam)
        self.LayoutControls(w, h)

    def selectVisibleAtIndex(self,index,add=0):
        if not add:
            current=self.SelItems[:]
            for i in current:
                self.itemsControl.SetItemState(i,0,commctrl.LVNI_SELECTED+commctrl.LVNI_FOCUSED)
        selectAt=self.itemsControl.GetTopIndex()+index-1
        self.itemsControl.SetItemState(selectAt,commctrl.LVNI_SELECTED+commctrl.LVNI_FOCUSED,255)
        self.itemsControl.EnsureVisible(selectAt, 0)
        self.itemsControl.SetFocus()

    def setCurrentAtTop(self):
        currentItems=self.SelItems[:]
        currentItems.sort()
        #for multiple selection set the first at the top, if it fits
        if (currentItems[-1]-currentItems[0])<self.itemsControl.GetCountPerPage():
            item=currentItems[0]
        else: #otherwise take the last one added
            item=self.SelItems[-1]
        bottom=item+self.itemsControl.GetCountPerPage()
        bottom=min(bottom,self.itemsControl.GetItemCount())
        self.itemsControl.EnsureVisible(bottom, 0)
        self.itemsControl.EnsureVisible(item, 0)
        self.itemsControl.SetFocus()

    def EndDialog(self, rc):
        self.StoreSelection()
        self._obj_.EndDialog(rc)
        return

    def StoreSelection(self):
        self.Selection=[]
        for i in self.SelItems:
            self.Selection.append(self.items[i])

    def OnOK(self):
        #self.setCurrentAtTop()
        #self.selectVisibleAtIndex(5)
        self.StoreSelection()
        self._obj_.OnOK()


class MultiListDialog(ListDialog):
    def __init__(self, title, List, colHeadings = None, size=(300,200,None), okButton='OK', resize=None):
        ListDialog.__init__(self, title, List, size=size[0:2],okButton=okButton, resize=resize)
        self.RefList = List
        if len(size)>2:
            self.colWidths=size[2]
        else:
            self.colWidths=None
        if colHeadings:
            self.colHeadings = colHeadings
        else:
            self.colHeadings = []
            for i in range(1,len(List[0])+1):
                self.colHeadings.append('item '+str(i))

    def StoreSelection(self):
        self.Selection=[]
        for i in self.SelItems:
            self.Selection.append(self.RefList[i][0])
        
    def FillList(self):
        index = 0
        size = self.GetWindowRect()
        width = size[2] - size[0] - (10) - win32api.GetSystemMetrics(win32con.SM_CXVSCROLL)
        numCols = len(self.colHeadings)        
        if self.colWidths:
            colw=self.colWidths
        else:
            colw=[0.20]
            for i in range(1,numCols): colw.append((1-colw[0])/(numCols-1))
        for col in self.colHeadings:
            itemDetails = (commctrl.LVCFMT_LEFT, width*colw[index], col, 0)
            self.itemsControl.InsertColumn(index, itemDetails)
            index = index + 1
        index = 0
        for items in self.items:
            #preceding spaces are stripped to facilitate list completion            
            itemText=str(items[0]).lstrip()
            index = self.itemsControl.InsertItem(index+1, itemText, 0)
            for itemno in range(1,numCols):
                item = items[itemno]
                self.itemsControl.SetItemText(index, itemno, str(item))


#############global functions######################
# def caseIndependentSort(something, other):
#     something, other= repr(something).lower(),repr(other).lower()
#     return cmp(something, other)
    
def BuildDlgList(Refs,reverse=0):
# takes a dictionary and builds a list with tuples for listcontrol construction
# first the key, then the value
    RL = []
    RefKeys=list(Refs.keys())
    if not reverse: RefKeys.sort(key=str.lower)
    for x in RefKeys:
        if reverse:
            RL.append((Refs[x],x))
        else:
            RL.append((x,Refs[x]))
    if reverse: RL.sort(key=str.lower)            
    return RL


defaultSize=(400,200)
#Globally define request id's 
RQID_References=1
RQID_Symbols=2
RQID_TCom=3
RQID_TEnv=4
RQID_FVarList=5
RQID_FVar=6
RQID_FRoutineList=7
RQID_FRoutine=8
RQID_FFunctionList=9
RQID_FFunction=10
RQID_FStatementList=11
RQID_FStructureList=12
RQID_FIntrinsicList=13
RQID_FAttributeList=14
RQID_FOperatorList=15
RQID_FParameterList=16
RQID_FModuleList=17


# QHQH
RequestFileName=dataDirectory+'\\request.bin'
ResultFileName=dataDirectory+'\\result.bin'

def DumpData(Data,FileName):
    File=open(FileName,'w')
    pickle.dump(Data, File)
    File.close()

def GetDumpedData(FileName):
    try:
        File=open(FileName,'r')
    except:
        return None
    Data=pickle.load(File)
    File.close()
    return Data


def DeferredSelectFromListDialog(List, Titles, defer, size=defaultSize):
    import natlink
    DumpData((List, Titles, defer, size), RequestFileName)
    SetMic('off')
    AppBringUp('Server',Exec=PythonServerExe,Args='/app DialogServer.py /listdialog')
    time.sleep(2)
    #try:
    #    print 3
    #does no longer generate an error when Serverapp not running? DNS 5?
    natlink.recognitionMimic(['Request','List','Dialog']) 
    #    print 32
    #except:
    #    print 4
    #    AppBringUp('Server',Exec=PythonServerExe,Args='/app DialogServer.py /listdialog')
    #    SetMic('on')                


def SelectFromListDialog(List, Titles, size=defaultSize, defer=None):
    if defer:
        r=GetDumpedData(ResultFileName)
##        if r and (r[0]==defer[1]):
##            os.remove(ResultFileName)
##            SetMic('on')
##            return r[1]
##        else:
        DeferredSelectFromListDialog(List, Titles, defer, size=size)
        return None
    dlg = MultiListDialog(str(Titles[0]), List, colHeadings=Titles[1:], size=size, resize=1)
    #SwitchToDD()        
    r=dlg.DoModal()
    #SwitchToNat()        
    if r==win32con.IDOK:
        return dlg.Selection
    return None

def SelectFromDictionary(Dictionary, Titles, size=defaultSize, reverse=0, defer=None):
    return SelectFromListDialog(BuildDlgList(Dictionary, reverse), Titles, size=size, defer=defer)


def ExtractofDictionary(Indices, Keys, Dictionary):
    Selected={}
    for i in Indices: Selected[Keys[i]]=Dictionary[Keys[i]]
    return Selected

def SelectFromPartDictionary(Indices, Keys, Dictionary, Titles, size=defaultSize, reverse=0, defer=None):
    Extracted=ExtractofDictionary(Indices, Keys, Dictionary)
    return SelectFromDictionary(Extracted, Titles, size=size, reverse=reverse, defer=defer)



def test2():
##    import D_TeX
##    RefList = BuildDlgList(D_TeX.Refs)
##  multilist from strange dictionary, example taken up by Quintijn
    import D_
    RefList = BuildDlgList(D_.Directories)
    dlg = MultiListDialog('Select', RefList, size=(400,200),okButton='Train', resize=1)
    if dlg.DoModal()==win32con.IDOK:
        return dlg.Selection
    return None

def test():
##    List=[(1),(2),(3),(4),(5)]#,(6),(7),(8),(9),(10),(11)]
    List=["foo", "bar two", "eggs three", "times four"]
    dlg = MultiListDialog('Select Special Grammars', List, 'Grammar')
    if dlg.DoModal()==win32con.IDOK:
        return dlg.Selection
    return None

def test_natspeak_on():
    #result = SelectFromListDialog(["foo","bar"], ["title 1", "title 2"], defer=1)
    result = SelectFromListDialog([["foo",1],["bar",2]], ["window title","title 1", "title 2"], defer=0)

def test_folders_list():
    """only display, background"""
    List=[r"C:\natlink\\unimacro", r"C:\documenten"]
    dlg = MultiListDialog('Say "recent file #"', List, ['directory'])
    hndle = dlg.CreateWindow()
    print('hnlde: %s'% hndle)

if __name__=='__main__':
    print(test())



#
# (unimacro - natlink macro wrapper/extensions)
# _number.py 
#  written by: Quintijn Hoogenboom (QH softwaretraining & advies)
#  August 2003//April 2022 (python3)
#
"""smart and reliable number dictation

the number part of the grammar was initially provided by Joel Gould in
his grammar "winspch.py". It is changed quite a bit, but essentially
all sorts of numbers can be dictated with his grammar.

For real use copy the things from his grammar into another grammar. See
for example "_lines.py" and "firefox_browsing.py".

This grammar shows the simplest "number" rule. For more examples, see "_number extended.py".

WARNING: do not have them BOTH switched on ("_number simple.py" and "_number extended.py")

QH september 2013: rewriting of the functions, ruling out optional command words. The optional word "and" has been removed
            (now say "three hundred twenty" instead of "three hundred and twenty")

further comments in _number extended.py. Also see the page "number grammar" on the Unimacro we
"""
#pylint:disable=C0115, C0116, W0613
from dtactions.unimacro import unimacroutils
from dtactions.sendkeys import sendkeys as keystroke
import unimacro.natlinkutilsbj as natbj
import natlink

thisGrammar = None
ancestor = natbj.IniGrammar
class ThisGrammar(ancestor):

    language = unimacroutils.getLanguage()

    #Step 1, choose one of next three grammar rules:
    # the <integer> rule comes from these grammar rules
    number_rules = natbj.numberGrammarTill999[language] # hundreds, including a long string of digits
    #number_rules = natbj.numberGrammarTill999999[language] # including thousands
    #number_rules = natbj.numberGrammar[language] #  including millions
    # the rules <integer> and <float> are already defined.
    name = "number simple"

    gramSpec = """
<testnumber> exported = Number <number>;
<number> = <integer> | Minus <integer> | <float> | Minus <float>;
"""+number_rules+"""
    """
    def __init__(self):
        self.minus = False
        self.number = False
        super().__init__()
  
    def initialize(self):
        self.switchOnOrOff() 

    def gotResultsInit(self, words, fullResults):
        # Step 2:
        # you can initialise variables here, eg self.minus = None, self.number = ''. Not necessary, but
        # in some grammars more appropriate than in the next function.
        pass
        
    def gotResults_testnumber(self, words, fullResults):
        # step 3: specify the number you are waiting for
        # check if the word Minus is present, and then wait for the number named "number"
        # note in this example self.number is non existent until collectNumber has been visited.
        # self.minus will only be set if the rule number is visited.
        self.waitForNumber('number')
        self.minus = False 

    def gotResults_number(self, words, fullResults):
        # step 3a: when in this rule, the word Minus (or synonym or translation) has been spoken..
        self.minus = True
 
    def gotResults(self,words,fullResults):
        # step 4, in gotResults collect the number (as a string):
        self.collectNumber() # setting self.number, see self.waitForNumber above
        print(f'number from the _number simple grammar: {self.number}')
        if self.minus:
            self.number = '-' + self.number
        # disable when testing through unittestIniGrammar.py (in unimacro_test directory):
        self.outputNumber(self.number)
            
                      
    def outputNumber(self, number):
        keystroke(number)
        #Step 5:
        # Here some extra postprocessing for different programs:
        prog = unimacroutils.getProgName()
        if prog in ['iexplore', 'firefox', 'chrome', 'safari']:
            keystroke('{tab}')
        elif prog in ['natspeak']:  # DragonPad
            keystroke('{enter}')
        elif prog in ['excel']:
            keystroke('{tab}')

# standard stuff Joel (adapted for possible empty gramSpec, QH, unimacro)
def unload():
    #pylint:disable=W0603
    global thisGrammar
    if thisGrammar:
        thisGrammar.unload()
    thisGrammar = None 

if __name__ == "__main__":
    ## interactive use, for debugging:
    natlink.natConnect()
    try:
        thisGrammar = ThisGrammar()
        # thisGrammar.startInifile()
        thisGrammar.initialize()
    finally:
        natlink.natDisconnect()
elif __name__.find('.') == -1:
    # standard startup when Dragon starts:
    thisGrammar = ThisGrammar()
    thisGrammar.initialize()



"""


(unimacro - natlink macro wrapper/extensions)
see http://qh.antenna.nl/unimacro/aboutunimacro.html for copyright note
    
      
"""
#pylint:disable=C0209
import unittest
from dtactions.unimacro import unimacroactions as actions
import natlink
import UnimacroTestHelpers

action = actions.doAction


class MessageTest(UnimacroTestHelpers.UnimacroTestHelpers):
    """Testing the Message and YesNo functions in 'actions.py'

    and the corresponding shorthand commands
    """    
    def setUp(self):
        pass


    def test_Should_handle_embedded_quotes_and_newlines(self):
        mes = 'Do you want to do the "handle_embedded_quotes_and_newlines" test?'
        result = actions.YesNo(mes, 'Starting the handle_embedded_quotes_and_newlines Test')
        if not result:
            print('handle_embedded_quotes_and_newlines (MessageTest) skipped')
            return
        mes1 = 'This has embedded newlines: \n and \\r characters: \r in it:\n' \
              'and embedded quotes: " and \'\n\n\n' \
              'Please answer OK'
        mes2 = 'This has embedded newlines: \n and \\r characters: \r in it:\n' \
              'and embedded quotes: " and \'\n\n\n' \
              'Please answer Yes if there are embedded newlines and quotes in this Dialog and in previous one'
        result = actions.Message(mes1, "\"'\n\rhandle_embedded_quotes_and_newlines (MessageTest)")
        result = actions.YesNo(mes2, "\"\"'\"\r\nhandle_embedded_quotes_and_newlines (MessageTest)")
        self.assertEqual(True, result, "you answered NO to test: %s"% self.squeeze_whitespace(mes2))

        
        

    def test_Should_give_a_message(self):
        mes = "Do you want to do the Message Box tests?"
        result = actions.YesNo(mes, "Starting the MessageTest (Message Box)")
        if not result:
            print('Message Box tests (MessageTest) skipped')
            return

##        for alert,icon in enumerate([16, 32, 48, 64]):
##            mes = '''This is a test message with\n\n\ticon=%s and\n
##                  \talert=%s\n\nplease answer "OK"'''% (icon, alert)
##            actions.Message(mes, "MessageTest", icon=icon, alert=alert)

        actions.Message('This is a the default test message\n\nPlease answer "OK"', "MessageTest")


        for alert, icon in enumerate(['information', 'warning', 'critical']):
            if alert:
                mes = '''This is a test message
                         \nicon=%s and
                         \nalert (number of bells ringing)=%s
                         \nPlease answer "OK"'''%(icon, alert)
            else:
                mes = '''This is a test message
                         \n\ticon=%s and
                         \n\tno alert (sound heard)
                         \nPlease answer "OK"'''%icon
            actions.Message(mes, "MessageTest", icon=icon, alert=alert)
        mes = '''Where all of the previous messages correct,
                 \nespecially the icons and the number of alerts?
              '''
        result = actions.YesNo(mes, "Evaluation of MessageTest", icon=icon, alert=alert)
        self.assertEqual(True, result, "you answered NO to test: %s"% self.squeeze_whitespace(mes))
    


    def test_Should_give_a_YesNo_box(self):

        mes = "Do you want to do the YesNo Box tests?"
        result = actions.YesNo(mes, "Starting the MessageTest (YesNo Box)")
        if not result:
            print('YesNo Box tests (MessageTest) skipped')
            return

        res = actions.YesNo('Please answer "Yes"', "MessageTest")
        self.assertTrue(res, "Result should be True when you really answered *Yes*")
        res = actions.YesNo("Please answer\n'No'", "MessageTest")
        self.assertFalse(res, "Result should be False when you really answered *No*")
        for alert, icon in enumerate(['query', 'warning', 'critical']):
            if alert:
                mes = '''This is a test question\n\nicon=%s and
                      \nalert(number of bells ringing)=%s\n\nIs this OK?'''%(icon, alert)
            else:
                mes = '''This is a test question\n\n\ticon=%s and
                      \n\tno alert (sound heard)\n\nIs this OK?'''%icon
            print('alerts: ', alert)
            result = actions.YesNo(mes, "MessageTest", icon=icon, alert=alert)
            self.assertEqual(True, result, "you answered NO to test: %s"% self.squeeze_whitespace(mes))


if __name__ == "__main__":
    natlink.natConnect()
    unittest.main()
    natlink.natDisconnect()



#
# Python Macro Language for Dragon NaturallySpeaking
#   (c) Copyright 1999 by Joel Gould
#   Portions (c) Copyright 1999 by Dragon Systems, Inc.
#
#   This script performs some basic tests of the NatLink system.  Dragon
#   NaturallySpeaking should be running with nothing in the editor window
#   (that you want to preserve) before these tests are run.
#   performed.
# from pathlib import Path
import unittest
from dtactions.unimacro import unimacroutils
from dtactions.unimacro import unimacroactions as actions
from natlinkcore import natlinkutils
import TestCaseWithHelpers

# unimacrodir = str(Path('./..').resolve())
# if unimacrodir not in sys.path:
#     sys.path.append(unimacrodir)

action = actions.doAction


##class UnimacroBasicTest(TestCaseWithHelpers.TestCaseWithHelpers):
class BasicTest(TestCaseWithHelpers.TestCaseWithHelpers):
      
    def setUp(self):
        pass
##        natlink.natConnect()
        

    def test_Something_in_unimacro(self):
        print('testing something')
        lang = unimacroutils.getLanguage()
        self.assert_equal("enx", lang, "testing should be done from an English speech profile, not: %s"% lang)

# no main statement, run from command in _unimacrotest.py.


from natlinkcore import natlinkutils
import natlink
import os
import os.path
import time

class ThisGrammar(natlinkutils.GrammarBase):

    gramSpec = """
<trayicon> exported = test tray icon;
        """

    def initialize(self):
        self.load(self.gramSpec)
        self.activateAll()
        #self.gotResults_trayicon(None, None)

    def gotBegin(self, moduleInfo):
        pass

#  starting message
    def gotResults_trayicon(self, words, fullResults):


        #iconDir = r'D:\natlink\unimacro\icons'
        #for name in ['repeat', 'repeat2', 'waiting', 'waiting2']:
        #    iconPath = os.path.join(iconDir, name+'.ico')
        #    print 'iconPath', iconPath
        #    natlink.setTrayIcon(iconPath)
        #    time.sleep(0.5)
        #
        ## remove the icon:
        #natlink.setTrayIcon()
        self.stopShowing = 0  # can be used for interrupt by clicking on the tray icon...
        self.directions = ['right', 'down', 'up', 'left']
        self.loops = 12
        self.currentLoop = 0
        self.currentDirection = 0
        natlink.setTimerCallback(self.doNextIcon, 500)  # each 500 milliseconds
        
    def doNextIcon(self):
        if self.stopShowing:
            print('caught callback, stop the looping')
            natlink.setTrayIcon()
            natlink.setTimerCallback(None)
            return
        if self.currentLoop >= self.loops:
            self.currentLoop = 0
            self.currentDirection += 1
            if self.currentDirection >= len(self.directions):
                print('ready all done, remove the icon and stop the timercallback')
                natlink.setTrayIcon()
                natlink.setTimerCallback(None)
        if self.currentLoop%2:
            iconName = '%s2'% self.directions[self.currentDirection]
        else:
            iconName = '%s'% self.directions[self.currentDirection]
        print('set trayIcon: %s'% iconName)
        natlink.setTrayIcon(iconName, 'test also click on icon %s'% iconName, self.onTrayIcon)
        
    def onTrayIcon(self, message):
        """calls back into this function if the tray icon is clicked upon
        """
        print('caught the onTrayIcon callback, clicked on the tray icon with button: %s'% message)
        # remove the icon:
        self.stopShowing = 1
        
# standard stuff Joel (adapted for possible empty gramSpec, QH, unimacro)
thisGrammar = ThisGrammar()
if thisGrammar.gramSpec:
    thisGrammar.initialize()
else:
    thisGrammar = None

def unload():
    #pylint:disable=W0603
    global thisGrammar
    if thisGrammar: thisGrammar.unload()
    thisGrammar = None

